@@ -24,6 +24,7 @@
 #include <glib/gi18n.h>
 #include <math.h>
 #include <stdlib.h>
+#include <string.h>
 
 #include "gpm-graph-widget.h"
 #include "gpm-info-data.h"
@@ -81,10 +82,6 @@ gpm_graph_widget_event_description (GpmGraphWidgetEvent event)
 		return _("LCD dim");
 	} else if (event == GPM_GRAPH_WIDGET_EVENT_SCREEN_RESUME) {
 		return _("LCD resume");
-	} else if (event == GPM_GRAPH_WIDGET_EVENT_DPMS_OFF) {
-		return _("DPMS off");
-	} else if (event == GPM_GRAPH_WIDGET_EVENT_DPMS_ON) {
-		return _("DPMS on");
 	} else if (event == GPM_GRAPH_WIDGET_EVENT_SUSPEND) {
 		return _("Suspend");
 	} else if (event == GPM_GRAPH_WIDGET_EVENT_RESUME) {
@@ -103,40 +100,77 @@ gpm_graph_widget_event_description (GpmGraphWidgetEvent event)
 }
 
 /**
- * gpm_graph_widget_event_colour:
+ * gpm_graph_widget_get_event_visual:
  * @event: The event type, e.g. GPM_GRAPH_WIDGET_EVENT_SUSPEND
  * Return value: a colout, e.g. GPM_GRAPH_WIDGET_COLOUR_DARK_BLUE
  **/
-GpmGraphWidgetColour
-gpm_graph_widget_event_colour (GpmGraphWidgetEvent event)
+void
+gpm_graph_widget_get_event_visual (GpmGraphWidgetEvent   event,
+				   GpmGraphWidgetColour *colour,
+				   GpmGraphWidgetShape  *shape)
 {
 	if (event == GPM_GRAPH_WIDGET_EVENT_ON_AC) {
-		return GPM_GRAPH_WIDGET_COLOUR_BLUE;
+		*colour = GPM_GRAPH_WIDGET_COLOUR_BLUE;
+		*shape = GPM_GRAPH_WIDGET_SHAPE_CIRCLE;
 	} else if (event == GPM_GRAPH_WIDGET_EVENT_ON_BATTERY) {
-		return GPM_GRAPH_WIDGET_COLOUR_DARK_BLUE;
+		*colour = GPM_GRAPH_WIDGET_COLOUR_DARK_BLUE;
+		*shape = GPM_GRAPH_WIDGET_SHAPE_CIRCLE;
 	} else if (event == GPM_GRAPH_WIDGET_EVENT_SCREEN_DIM) {
-		return GPM_GRAPH_WIDGET_COLOUR_CYAN;
+		*colour = GPM_GRAPH_WIDGET_COLOUR_YELLOW;
+		*shape = GPM_GRAPH_WIDGET_SHAPE_SQUARE;
 	} else if (event == GPM_GRAPH_WIDGET_EVENT_SCREEN_RESUME) {
-		return GPM_GRAPH_WIDGET_COLOUR_DARK_CYAN;
-	} else if (event == GPM_GRAPH_WIDGET_EVENT_DPMS_OFF) {
-		return GPM_GRAPH_WIDGET_COLOUR_DARK_YELLOW;
-	} else if (event == GPM_GRAPH_WIDGET_EVENT_DPMS_ON) {
-		return GPM_GRAPH_WIDGET_COLOUR_YELLOW;
+		*colour = GPM_GRAPH_WIDGET_COLOUR_DARK_YELLOW;
+		*shape = GPM_GRAPH_WIDGET_SHAPE_SQUARE;
 	} else if (event == GPM_GRAPH_WIDGET_EVENT_SUSPEND) {
-		return GPM_GRAPH_WIDGET_COLOUR_RED;
+		*colour = GPM_GRAPH_WIDGET_COLOUR_RED;
+		*shape = GPM_GRAPH_WIDGET_SHAPE_DIAMOND;
 	} else if (event == GPM_GRAPH_WIDGET_EVENT_RESUME) {
-		return GPM_GRAPH_WIDGET_COLOUR_DARK_RED;
+		*colour = GPM_GRAPH_WIDGET_COLOUR_DARK_RED;
+		*shape = GPM_GRAPH_WIDGET_SHAPE_DIAMOND;
 	} else if (event == GPM_GRAPH_WIDGET_EVENT_HIBERNATE) {
-		return GPM_GRAPH_WIDGET_COLOUR_MAGENTA;
+		*colour = GPM_GRAPH_WIDGET_COLOUR_MAGENTA;
+		*shape = GPM_GRAPH_WIDGET_SHAPE_DIAMOND;
 	} else if (event == GPM_GRAPH_WIDGET_EVENT_LID_CLOSED) {
-		return GPM_GRAPH_WIDGET_COLOUR_GREEN;
+		*colour = GPM_GRAPH_WIDGET_COLOUR_GREEN;
+		*shape = GPM_GRAPH_WIDGET_SHAPE_TRIANGLE;
 	} else if (event == GPM_GRAPH_WIDGET_EVENT_LID_OPENED) {
-		return GPM_GRAPH_WIDGET_COLOUR_DARK_GREEN;
+		*colour = GPM_GRAPH_WIDGET_COLOUR_DARK_GREEN;
+		*shape = GPM_GRAPH_WIDGET_SHAPE_TRIANGLE;
 	} else if (event == GPM_GRAPH_WIDGET_EVENT_NOTIFICATION) {
-		return GPM_GRAPH_WIDGET_COLOUR_GREY;
+		*colour = GPM_GRAPH_WIDGET_COLOUR_GREY;
+		*shape = GPM_GRAPH_WIDGET_SHAPE_CIRCLE;
 	} else {
-		return GPM_GRAPH_WIDGET_COLOUR_DEFAULT;
+		*colour = GPM_GRAPH_WIDGET_COLOUR_DEFAULT;
+		*shape = GPM_GRAPH_WIDGET_SHAPE_CIRCLE;
 	}
+	return;
+}
+
+/**
+ * gpm_graph_widget_string_to_axis_type:
+ * @graph: This graph class instance
+ * @type: The axis type, e.g. "percentage"
+ *
+ * Return value: The enumerated axis type
+ **/
+GpmGraphWidgetAxisType
+gpm_graph_widget_string_to_axis_type (const gchar *type)
+{
+	GpmGraphWidgetAxisType ret;
+	g_return_val_if_fail (type != NULL, GPM_GRAPH_WIDGET_TYPE_INVALID);
+
+	ret = GPM_GRAPH_WIDGET_TYPE_INVALID;
+	if (strcmp (type, "percentage") == 0) {
+		ret = GPM_GRAPH_WIDGET_TYPE_PERCENTAGE;
+	} else if (strcmp (type, "time") == 0) {
+		ret = GPM_GRAPH_WIDGET_TYPE_TIME;
+	} else if (strcmp (type, "power") == 0) {
+		ret = GPM_GRAPH_WIDGET_TYPE_POWER;
+	} else if (strcmp (type, "voltage") == 0) {
+		ret = GPM_GRAPH_WIDGET_TYPE_VOLTAGE;
+	}
+	
+	return ret;
 }
 
 /**
@@ -325,7 +359,7 @@ gpm_graph_widget_set_events (GpmGraphWidget *graph, GList *list)
  *
  * Unit is:
  * GPM_GRAPH_WIDGET_TYPE_TIME:		seconds
- * GPM_GRAPH_WIDGET_TYPE_RATE: 	mWh (not mAh)
+ * GPM_GRAPH_WIDGET_TYPE_POWER: 	mWh (not mAh)
  * GPM_GRAPH_WIDGET_TYPE_PERCENTAGE:	%
  *
  * Return value: a string value depending on the axis type and the value.
@@ -341,23 +375,33 @@ gpm_get_axis_label (GpmGraphWidgetAxisType axis, gint value)
 		minutes =  minutes - (hours * 60);
 		if (hours > 0) {
 			if (minutes == 0) {
-				text = g_strdup_printf ("%ih", hours);
+				/*Translators: This is %i hours*/
+				text = g_strdup_printf (_("%ih"), hours);
 			} else {
-				text = g_strdup_printf ("%ih%02i", hours, minutes);
+				/*Translators: This is %i hours %02i minutes*/
+				text = g_strdup_printf (_("%ih%02i"), hours, minutes);
 			}
 		} else if (minutes > 0) {
 			if (seconds == 0) {
-				text = g_strdup_printf ("%2im", minutes);
+				/*Translators: This is %2i minutes*/
+				text = g_strdup_printf (_("%2im"), minutes);
 			} else {
-				text = g_strdup_printf ("%2im%02i", minutes, seconds);
+				/*Translators: This is %2i minutes %02i seconds*/
+				text = g_strdup_printf (_("%2im%02i"), minutes, seconds);
 			}
 		} else {
-			text = g_strdup_printf ("%2is", seconds);
+			/*Translators: This is %2i seconds*/
+			text = g_strdup_printf (_("%2is"), seconds);
 		}
 	} else if (axis == GPM_GRAPH_WIDGET_TYPE_PERCENTAGE) {
-		text = g_strdup_printf ("%i%%", value);
-	} else if (axis == GPM_GRAPH_WIDGET_TYPE_RATE) {
-		text = g_strdup_printf ("%iW", value / 1000);
+		/*Translators: This is %i Percentage*/
+		text = g_strdup_printf (_("%i%%"), value);
+	} else if (axis == GPM_GRAPH_WIDGET_TYPE_POWER) {
+		/*Translators: This is %i Watts*/
+		text = g_strdup_printf (_("%iW"), value / 1000);
+	} else if (axis == GPM_GRAPH_WIDGET_TYPE_VOLTAGE) {
+		/*Translators: This is %i Volts*/
+		text = g_strdup_printf (_("%iV"), value / 1000);
 	} else {
 		text = g_strdup_printf ("%i??", value);
 	}
@@ -582,11 +626,16 @@ gpm_graph_widget_auto_range (GpmGraphWidget *graph)
 		graph->priv->stop_x = 100;
 	} else if (graph->priv->axis_x == GPM_GRAPH_WIDGET_TYPE_TIME) {
 		graph->priv->stop_x = (((biggest_x - smallest_x) / (10 * 60)) + 1) * (10 * 60) + smallest_x;
-	} else if (graph->priv->axis_x == GPM_GRAPH_WIDGET_TYPE_RATE) {
+	} else if (graph->priv->axis_x == GPM_GRAPH_WIDGET_TYPE_POWER) {
 		graph->priv->stop_x = (((biggest_x - smallest_x) / 10000) + 2) * 10000 + smallest_x;
 		if (graph->priv->stop_x < 10000) {
 			graph->priv->stop_x = 10000 + smallest_x;
 		}
+	} else if (graph->priv->axis_x == GPM_GRAPH_WIDGET_TYPE_VOLTAGE) {
+		graph->priv->stop_x = (((biggest_x - smallest_x) / 1000) + 2) * 1000 + smallest_x;
+		if (graph->priv->stop_x < 1000) {
+			graph->priv->stop_x = 1000 + smallest_x;
+		}
 	} else {
 		graph->priv->stop_x = ((biggest_x / 10) + 1) * 10 + smallest_x;
 	}
@@ -604,12 +653,18 @@ gpm_graph_widget_auto_range (GpmGraphWidget *graph)
 		if (graph->priv->stop_y < 60) {
 			graph->priv->stop_y = 60;
 		}
-	} else if (graph->priv->axis_y == GPM_GRAPH_WIDGET_TYPE_RATE) {
+	} else if (graph->priv->axis_y == GPM_GRAPH_WIDGET_TYPE_POWER) {
 		graph->priv->start_y = 0;
 		graph->priv->stop_y = ((biggest_y / 10000) + 2) * 10000;
 		if (graph->priv->stop_y < 10000) {
 			graph->priv->stop_y = 10000;
 		}
+	} else if (graph->priv->axis_y == GPM_GRAPH_WIDGET_TYPE_VOLTAGE) {
+		graph->priv->start_y = 0;
+		graph->priv->stop_y = ((biggest_y / 1000) + 2) * 1000;
+		if (graph->priv->stop_y < 1000) {
+			graph->priv->stop_y = 1000;
+		}
 	} else {
 		graph->priv->start_y = 0;
 		graph->priv->stop_y = ((biggest_y / 10) + 1) * 10;
@@ -672,19 +727,78 @@ gpm_graph_widget_set_colour (cairo_t *cr, GpmGraphWidgetColour colour)
  * @x: The X-coordinate for the center
  * @y: The Y-coordinate for the center
  * @colour: The colour enum
+ * @shape: The shape enum
  *
  * Draw the dot on the graph of a specified colour
  **/
 static void
-gpm_graph_widget_draw_dot (cairo_t *cr, gfloat x, gfloat y, GpmGraphWidgetColour colour)
+gpm_graph_widget_draw_dot (cairo_t             *cr,
+			   gfloat               x,
+			   gfloat               y,
+			   GpmGraphWidgetColour colour,
+			   GpmGraphWidgetShape  shape)
 {
-	cairo_arc (cr, (gint)x + 0.5f, (gint)y + 0.5f, 4, 0, 2*M_PI);
-	gpm_graph_widget_set_colour (cr, colour);
-	cairo_fill (cr);
-	cairo_arc (cr, (gint)x + 0.5f, (gint)y + 0.5f, 4, 0, 2*M_PI);
-	cairo_set_source_rgb (cr, 0, 0, 0);
-	cairo_set_line_width (cr, 1);
-	cairo_stroke (cr);
+	gfloat width;
+	if (shape == GPM_GRAPH_WIDGET_SHAPE_CIRCLE) {
+		/* circle */
+		cairo_arc (cr, (gint)x + 0.5f, (gint)y + 0.5f, 4, 0, 2*M_PI);
+		gpm_graph_widget_set_colour (cr, colour);
+		cairo_fill (cr);
+		cairo_arc (cr, (gint)x + 0.5f, (gint)y + 0.5f, 4, 0, 2*M_PI);
+		cairo_set_source_rgb (cr, 0, 0, 0);
+		cairo_set_line_width (cr, 1);
+		cairo_stroke (cr);
+	} else if (shape == GPM_GRAPH_WIDGET_SHAPE_SQUARE) {
+		/* box */
+		width = 8.0;
+		cairo_rectangle (cr, (gint)x + 0.5f - (width/2), (gint)y + 0.5f - (width/2), width, width);
+		gpm_graph_widget_set_colour (cr, colour);
+		cairo_fill (cr);
+		cairo_rectangle (cr, (gint)x + 0.5f - (width/2), (gint)y + 0.5f - (width/2), width, width);
+		cairo_set_source_rgb (cr, 0, 0, 0);
+		cairo_set_line_width (cr, 1);
+		cairo_stroke (cr);
+	} else if (shape == GPM_GRAPH_WIDGET_SHAPE_DIAMOND) {
+		/* diamond */
+		width = 4.0;
+		cairo_new_path (cr);
+		cairo_move_to (cr, x+0.5, y-width+0.5);
+		cairo_line_to (cr, x+width+0.5, y+0.5);
+		cairo_line_to (cr, x+0.5, y+width+0.5);
+		cairo_line_to (cr, x-width+0.5, y+0.5);
+		cairo_close_path (cr);
+		gpm_graph_widget_set_colour (cr, colour);
+		cairo_fill (cr);
+		cairo_new_path (cr);
+		cairo_move_to (cr, x+0.5, y-width+0.5);
+		cairo_line_to (cr, x+width+0.5, y+0.5);
+		cairo_line_to (cr, x+0.5, y+width+0.5);
+		cairo_line_to (cr, x-width+0.5, y+0.5);
+		cairo_close_path (cr);
+		cairo_set_source_rgb (cr, 0, 0, 0);
+		cairo_set_line_width (cr, 1);
+		cairo_stroke (cr);
+	} else if (shape == GPM_GRAPH_WIDGET_SHAPE_TRIANGLE) {
+		/* triangle */
+		width = 4.0;
+		cairo_new_path (cr);
+		cairo_move_to (cr, x+0.5, y-width+0.5);
+		cairo_line_to (cr, x+width+0.5, y+width+0.5-1.0);
+		cairo_line_to (cr, x-width+0.5, y+width+0.5-1.0);
+		cairo_close_path (cr);
+		gpm_graph_widget_set_colour (cr, colour);
+		cairo_fill (cr);
+		cairo_new_path (cr);
+		cairo_move_to (cr, x+0.5, y-width+0.5);
+		cairo_line_to (cr, x+width+0.5, y+width+0.5-1.0);
+		cairo_line_to (cr, x-width+0.5, y+width+0.5-1.0);
+		cairo_close_path (cr);
+		cairo_set_source_rgb (cr, 0, 0, 0);
+		cairo_set_line_width (cr, 1);
+		cairo_stroke (cr);
+	} else {
+		gpm_warning ("shape %i not recognised!", shape);
+	}
 }
 
 /**
@@ -829,6 +943,8 @@ gpm_graph_widget_draw_line (GpmGraphWidget *graph, cairo_t *cr)
 		gint prevpos = -1;
 		GpmInfoDataPoint *point_this = NULL;
 		GpmInfoDataPoint *point_last = NULL;
+		GpmGraphWidgetColour colour;
+		GpmGraphWidgetShape shape;
 
 		/* we track the list so we can put the point on the line */
 		GList *l2 = graph->priv->list;
@@ -864,7 +980,10 @@ gpm_graph_widget_draw_line (GpmGraphWidget *graph, cairo_t *cr)
 			newy -= (8 * previous_point);
 			/* only do the event dot, if it's going to fit on the graph */
 			if (eventdata->time > graph->priv->start_x) {
-				gpm_graph_widget_draw_dot (cr, newx, newy, eventdata->colour);
+				gpm_graph_widget_get_event_visual (eventdata->value,
+								   &colour, &shape);
+				gpm_graph_widget_draw_dot (cr, newx, newy,
+							   colour, shape);
 			}
 			prevpos = newx;
 		}
@@ -918,13 +1037,15 @@ gpm_graph_widget_draw_legend (cairo_t *cr,
 	gint y_count;
 	gint a;
 	GpmGraphWidgetColour colour;
+	GpmGraphWidgetShape shape;
 
 	gpm_graph_widget_draw_bounding_box (cr, x, y, width, height);
 	y_count = y + 10;
 	for (a=0; a<GPM_GRAPH_WIDGET_EVENT_LAST; a++) {
 		desc = gpm_graph_widget_event_description (a);
-		colour = gpm_graph_widget_event_colour (a);
-		gpm_graph_widget_draw_dot (cr, x + 8, y_count, colour);
+		gpm_graph_widget_get_event_visual (a, &colour, &shape);
+		gpm_graph_widget_draw_dot (cr, x + 8, y_count,
+					   colour, shape);
 		cairo_move_to (cr, x + 8 + 10, y_count + 3);
 		cairo_show_text (cr, desc);
 		y_count = y_count + GPM_GRAPH_WIDGET_LEGEND_SPACING;
