@@ -96,36 +96,6 @@ gpm_hal_is_running (GpmHal *hal)
 	return running;
 }
 
-/* we have to be clever, as hal can pass back two types of errors, and we have
-   to ignore dbus timeouts */
-static gboolean
-gpm_hal_handle_error (guint ret, GError *error, const gchar *method)
-{
-	gboolean retval = TRUE;
-
-	g_return_val_if_fail (method != NULL, FALSE);
-
-	if (error) {
-		/* DBUS might time out, which is okay. We can remove this code
-		   when the dbus glib bindings are fixed. See #332888 */
-		if (g_error_matches (error, DBUS_GERROR, DBUS_GERROR_NO_REPLY)) {
-			gpm_debug ("DBUS timed out, but recovering");
-			retval = TRUE;
-		} else {
-			gpm_warning ("%s failed\n(%s)",
-				     method,
-				     error->message);
-			retval = FALSE;
-		}
-		g_error_free (error);
-	} else if (ret != 0) {
-		/* we might not get an error set */
-		gpm_warning ("%s failed (Unknown error)", method);
-		retval = FALSE;
-	}
-	return retval;
-}
-
 /**
  * gpm_hal_device_rescan:
  *
@@ -138,10 +108,10 @@ gpm_hal_handle_error (guint ret, GError *error, const gchar *method)
 static gboolean
 gpm_hal_device_rescan (GpmHal *hal, const gchar *udi)
 {
-	gint ret = 0;
+	gint retval = 0;
 	DBusGProxy *proxy;
 	GError *error = NULL;
-	gboolean retval;
+	gboolean ret;
 
 	g_return_val_if_fail (GPM_IS_HAL (hal), FALSE);
 	g_return_val_if_fail (udi != NULL, FALSE);
@@ -151,14 +121,20 @@ gpm_hal_device_rescan (GpmHal *hal, const gchar *udi)
 					   HAL_DBUS_SERVICE,
 					   udi,
 					   HAL_DBUS_INTERFACE_DEVICE);
-	dbus_g_proxy_call (proxy, "Rescan", &error,
-			   G_TYPE_INVALID,
-			   G_TYPE_BOOLEAN, &ret, G_TYPE_INVALID);
-	retval = gpm_hal_handle_error (ret, error, "rescan");
-
+	ret = dbus_g_proxy_call (proxy, "Rescan", &error,
+				 G_TYPE_INVALID,
+				 G_TYPE_BOOLEAN, &retval, G_TYPE_INVALID);
+	if (error) {
+		gpm_debug ("ERROR: %s", error->message);
+		g_error_free (error);
+	}
+	if (ret == FALSE || retval != 0) {
+		/* abort as the DBUS method failed */
+		gpm_warning ("Rescan failed!");
+		return FALSE;
+	}
 	g_object_unref (G_OBJECT (proxy));
-	return retval;
-
+	return TRUE;
 }
 
 /**
@@ -208,7 +184,7 @@ gpm_hal_device_get_bool (GpmHal      *hal,
 {
 	DBusGProxy *proxy;
 	GError *error = NULL;
-	gboolean retval;
+	gboolean ret;
 
 	g_return_val_if_fail (GPM_IS_HAL (hal), FALSE);
 	g_return_val_if_fail (udi != NULL, FALSE);
@@ -219,19 +195,23 @@ gpm_hal_device_get_bool (GpmHal      *hal,
 					       HAL_DBUS_SERVICE,
 					       udi,
 					       HAL_DBUS_INTERFACE_DEVICE);
-	retval = TRUE;
-	if (dbus_g_proxy_call (proxy, "GetPropertyBoolean", &error,
-			       G_TYPE_STRING, key, G_TYPE_INVALID,
-			       G_TYPE_BOOLEAN, value, G_TYPE_INVALID) == FALSE) {
-		if (error) {
-			gpm_debug ("%s", error->message);
-			g_error_free (error);
-		}
+	ret = dbus_g_proxy_call (proxy, "GetPropertyBoolean", &error,
+				 G_TYPE_STRING, key,
+				 G_TYPE_INVALID,
+				 G_TYPE_BOOLEAN, value,
+				 G_TYPE_INVALID);
+	if (error) {
+		gpm_debug ("ERROR: %s", error->message);
+		g_error_free (error);
+	}
+	if (ret == FALSE) {
+		/* abort as the DBUS method failed */
 		*value = FALSE;
-		retval = FALSE;
+		gpm_debug ("GetPropertyBoolean failed!");
+		return FALSE;
 	}
 	g_object_unref (G_OBJECT (proxy));
-	return retval;
+	return TRUE;
 }
 
 /**
@@ -253,7 +233,7 @@ gpm_hal_device_get_string (GpmHal      *hal,
 {
 	DBusGProxy *proxy = NULL;
 	GError *error = NULL;
-	gboolean retval;
+	gboolean ret;
 
 	g_return_val_if_fail (GPM_IS_HAL (hal), FALSE);
 	g_return_val_if_fail (udi != NULL, FALSE);
@@ -264,19 +244,23 @@ gpm_hal_device_get_string (GpmHal      *hal,
 					       HAL_DBUS_SERVICE,
 					       udi,
 					       HAL_DBUS_INTERFACE_DEVICE);
-	retval = TRUE;
-	if (dbus_g_proxy_call (proxy, "GetPropertyString", &error,
-			       G_TYPE_STRING, key, G_TYPE_INVALID,
-			       G_TYPE_STRING, value, G_TYPE_INVALID) == FALSE) {
-		if (error) {
-			gpm_debug ("%s", error->message);
-			g_error_free (error);
-		}
+	ret = dbus_g_proxy_call (proxy, "GetPropertyString", &error,
+				 G_TYPE_STRING, key,
+				 G_TYPE_INVALID,
+				 G_TYPE_STRING, value,
+				 G_TYPE_INVALID);
+	if (error) {
+		gpm_debug ("ERROR: %s", error->message);
+		g_error_free (error);
+	}
+	if (ret == FALSE) {
+		/* abort as the DBUS method failed */
 		*value = NULL;
-		retval = FALSE;
+		gpm_debug ("GetPropertyString failed!");
+		return FALSE;
 	}
 	g_object_unref (G_OBJECT (proxy));
-	return retval;
+	return TRUE;
 }
 
 /**
@@ -296,7 +280,7 @@ gpm_hal_device_get_int (GpmHal      *hal,
 {
 	DBusGProxy *proxy;
 	GError *error = NULL;
-	gboolean retval;
+	gboolean ret;
 
 	g_return_val_if_fail (GPM_IS_HAL (hal), FALSE);
 	g_return_val_if_fail (udi != NULL, FALSE);
@@ -307,19 +291,43 @@ gpm_hal_device_get_int (GpmHal      *hal,
 					       HAL_DBUS_SERVICE,
 					       udi,
 					       HAL_DBUS_INTERFACE_DEVICE);
-	retval = TRUE;
-	if (dbus_g_proxy_call (proxy, "GetPropertyInteger", &error,
-				G_TYPE_STRING, key, G_TYPE_INVALID,
-				G_TYPE_INT, value, G_TYPE_INVALID) == FALSE) {
-		if (error) {
-			gpm_debug ("%s", error->message);
-			g_error_free (error);
-		}
+	ret = dbus_g_proxy_call (proxy, "GetPropertyInteger", &error,
+				 G_TYPE_STRING, key,
+				 G_TYPE_INVALID,
+				 G_TYPE_INT, value,
+				 G_TYPE_INVALID);
+	if (error) {
+		gpm_debug ("ERROR: %s", error->message);
+		g_error_free (error);
+	}
+	if (ret == FALSE) {
+		/* abort as the DBUS method failed */
 		*value = 0;
-		retval = FALSE;
+		gpm_debug ("GetPropertyInteger failed!");
+		return FALSE;
 	}
 	g_object_unref (G_OBJECT (proxy));
-	return retval;
+	return TRUE;
+}
+
+/**
+ * gpm_hal_device_get_uint:
+ *
+ * HAL has no concept of a UINT, only INT
+ **/
+gboolean
+gpm_hal_device_get_uint (GpmHal      *hal,
+			 const gchar *udi,
+			 const gchar *key,
+			 guint       *value)
+{
+	gint tvalue;
+	gboolean ret;
+
+	/* bodge */
+	ret = gpm_hal_device_get_int (hal, udi, key, &tvalue);
+	*value = (guint) tvalue;
+	return ret;
 }
 
 /**
@@ -337,7 +345,7 @@ gpm_hal_device_find_capability (GpmHal      *hal,
 {
 	DBusGProxy *proxy = NULL;
 	GError *error = NULL;
-	gboolean retval;
+	gboolean ret;
 
 	g_return_val_if_fail (GPM_IS_HAL (hal), FALSE);
 	g_return_val_if_fail (capability != NULL, FALSE);
@@ -347,19 +355,69 @@ gpm_hal_device_find_capability (GpmHal      *hal,
 					       HAL_DBUS_SERVICE,
 					       HAL_DBUS_PATH_MANAGER,
 					       HAL_DBUS_INTERFACE_MANAGER);
-	retval = TRUE;
-	if (dbus_g_proxy_call (proxy, "FindDeviceByCapability", &error,
-			        G_TYPE_STRING, capability, G_TYPE_INVALID,
-			        G_TYPE_STRV, value, G_TYPE_INVALID) == FALSE) {
-		if (error) {
-			gpm_debug ("%s", error->message);
-			g_error_free (error);
-		}
+	ret = dbus_g_proxy_call (proxy, "FindDeviceByCapability", &error,
+				 G_TYPE_STRING, capability,
+				 G_TYPE_INVALID,
+				 G_TYPE_STRV, value,
+				 G_TYPE_INVALID);
+	if (error) {
+		gpm_debug ("ERROR: %s", error->message);
+		g_error_free (error);
+	}
+	if (ret == FALSE) {
+		/* abort as the DBUS method failed */
 		*value = NULL;
-		retval = FALSE;
+		gpm_warning ("FindDeviceByCapability failed!");
+		return FALSE;
 	}
 	g_object_unref (G_OBJECT (proxy));
-	return retval;
+	return TRUE;
+}
+
+/**
+ * gpm_hal_device_has_capability:
+ *
+ * @hal: This hal class instance
+ * @capability: The capability, e.g. "battery"
+ * @value: return value, passed by ref
+ * Return value: TRUE for success, FALSE for failure
+ **/
+gboolean
+gpm_hal_device_has_capability (GpmHal      *hal,
+			       const gchar *udi,
+			       const gchar *capability,
+			       gboolean    *has_capability)
+{
+	DBusGProxy *proxy = NULL;
+	GError *error = NULL;
+	gboolean ret;
+
+	g_return_val_if_fail (GPM_IS_HAL (hal), FALSE);
+	g_return_val_if_fail (udi != NULL, FALSE);
+	g_return_val_if_fail (capability != NULL, FALSE);
+	g_return_val_if_fail (has_capability != NULL, FALSE);
+
+	proxy = dbus_g_proxy_new_for_name (hal->priv->connection,
+					   HAL_DBUS_SERVICE,
+					   udi,
+					   HAL_DBUS_INTERFACE_DEVICE);
+	ret = dbus_g_proxy_call (proxy, "QueryCapability", &error,
+				 G_TYPE_STRING, udi,
+				 G_TYPE_INVALID,
+				 G_TYPE_BOOLEAN, has_capability,
+				 G_TYPE_INVALID);
+	if (error) {
+		gpm_debug ("ERROR: %s", error->message);
+		g_error_free (error);
+	}
+	if (ret == FALSE) {
+		/* abort as the DBUS method failed */
+		*has_capability = FALSE;
+		gpm_warning ("QueryCapability failed!");
+		return FALSE;
+	}
+	g_object_unref (G_OBJECT (proxy));
+	return TRUE;
 }
 
 /**
@@ -543,11 +601,13 @@ watch_device_properties_modified_cb (DBusGProxy *proxy,
  *
  * @udi: The HAL UDI
  *
- * Watch the specified device, so it emits device-property-modified
+ * Watch the specified device, so it emits device-property-modified and
+ * adds to the gpm cache so we don't get asked to add it again.
  */
 gboolean
 gpm_hal_device_watch_propery_modified (GpmHal      *hal,
-				       const gchar *udi)
+				       const gchar *udi,
+				       gboolean     force)
 {
 	DBusGProxy *proxy;
 	GError     *error = NULL;
@@ -558,7 +618,7 @@ gpm_hal_device_watch_propery_modified (GpmHal      *hal,
 
 	/* Check we are not already monitoring this device */
 	proxy = g_hash_table_lookup (hal->priv->watch_device_property_modified, udi);
-	if (proxy != NULL) {
+	if (force == FALSE && proxy != NULL) {
 		gpm_debug ("Device is already being watched for PropertyModified: %s", udi);
 		return FALSE;
 	}
@@ -624,7 +684,8 @@ watch_device_condition_cb (DBusGProxy  *proxy,
  */
 gboolean
 gpm_hal_device_watch_condition (GpmHal      *hal,
-				const gchar *udi)
+				const gchar *udi,
+				gboolean     force)
 {
 	DBusGProxy *proxy;
 	GError     *error = NULL;
@@ -634,7 +695,7 @@ gpm_hal_device_watch_condition (GpmHal      *hal,
 
 	/* Check we are not already monitoring this device */
 	proxy = g_hash_table_lookup (hal->priv->watch_device_condition, udi);
-	if (proxy != NULL) {
+	if (force == FALSE && proxy != NULL) {
 		gpm_debug ("Device is already being watched for NewCondition: %s", udi);
 		return FALSE;
 	}
@@ -651,7 +712,6 @@ gpm_hal_device_watch_condition (GpmHal      *hal,
 	}
 	g_hash_table_insert (hal->priv->watch_device_condition, g_strdup (udi), proxy);
 
-
 	dbus_g_object_register_marshaller (gpm_marshal_VOID__STRING_STRING,
 					   G_TYPE_NONE, G_TYPE_STRING, G_TYPE_STRING,
 					   G_TYPE_INVALID);
@@ -890,6 +950,40 @@ gpm_hal_new_capability_cb (DBusGProxy  *proxy,
 	g_signal_emit (hal, signals [NEW_CAPABILITY], 0, udi, capability);
 }
 
+/**
+ * reattach_property_modified_in_hash:
+ *
+ * @udi: The HAL UDI
+ *
+ * HashFunc so we can remove all the device-propery-modified devices
+ */
+static void
+reattach_property_modified_in_hash (const gchar *udi,
+				    gpointer     value,
+				    GpmHal      *hal)
+{
+	/* force the new proxy as DBUS has invalidaded the old ones */
+	g_debug ("reattach property-modified %s", udi);
+	gpm_hal_device_watch_propery_modified (hal, udi, TRUE);
+}
+
+/**
+ * reattach_condition_in_hash:
+ *
+ * @udi: The HAL UDI
+ *
+ * HashFunc so we can remove all the device-propery-modified devices
+ */
+static void
+reattach_condition_in_hash (const gchar *udi,
+				    gpointer     value,
+				    GpmHal      *hal)
+{
+	/* force the new proxy as DBUS has invalidaded the old ones */
+	g_debug ("reattach condition %s", udi);
+	gpm_hal_device_watch_condition (hal, udi, TRUE);
+}
+
 /**
  * gpm_hal_proxy_connect_more:
  *
@@ -907,7 +1001,7 @@ gpm_hal_proxy_connect_more (GpmHal *hal)
 
 	proxy = gpm_proxy_get_proxy (hal->priv->gproxy);
 	if (proxy == NULL) {
-		g_warning ("not connected");
+		gpm_warning ("not connected");
 		return FALSE;
 	}	
 
@@ -929,7 +1023,15 @@ gpm_hal_proxy_connect_more (GpmHal *hal)
 	dbus_g_proxy_connect_signal (proxy, "NewCapability",
 				     G_CALLBACK (gpm_hal_new_capability_cb), hal, NULL);
 
-/*emit daemon stop!*/
+	/* we need to re-register anything in:
+	   - hal->priv->watch_device_condition
+	   - hal->priv->watch_property_modified
+	 */
+	g_hash_table_foreach (hal->priv->watch_device_property_modified,
+			      (GHFunc) reattach_property_modified_in_hash, hal);
+	g_hash_table_foreach (hal->priv->watch_device_condition,
+			      (GHFunc) reattach_condition_in_hash, hal);
+
 	return TRUE;
 }
 
@@ -950,7 +1052,7 @@ gpm_hal_proxy_disconnect_more (GpmHal *hal)
 
 	proxy = gpm_proxy_get_proxy (hal->priv->gproxy);
 	if (proxy == NULL) {
-		g_warning ("not connected");
+		gpm_warning ("not connected");
 		return FALSE;
 	}	
 
@@ -974,7 +1076,6 @@ proxy_status_cb (DBusGProxy *proxy,
 		 GpmHal     *hal)
 {
 	g_return_if_fail (GPM_IS_HAL (hal));
-
 	if (status) {
 		gpm_hal_proxy_connect_more (hal);
 	} else {
@@ -1009,8 +1110,7 @@ gpm_hal_init (GpmHal *hal)
 				  HAL_DBUS_INTERFACE_MANAGER);
 
 	g_signal_connect (hal->priv->gproxy, "proxy-status",
-			  G_CALLBACK (proxy_status_cb),
-			  hal);
+			  G_CALLBACK (proxy_status_cb), hal);
 
 	hal->priv->watch_device_property_modified = g_hash_table_new (g_str_hash, g_str_equal);
 	hal->priv->watch_device_condition = g_hash_table_new (g_str_hash, g_str_equal);
