@@ -43,6 +43,7 @@
 
 #include "gpm-common.h"
 #include "gpm-prefs.h"
+#include "gpm-gconf.h"
 #include "gpm-screensaver.h"
 #include "gpm-networkmanager.h"
 
@@ -50,7 +51,8 @@
 #include "gpm-hal-monitor.h"
 #include "gpm-hal-cpufreq.h"
 #include "gpm-hal-power.h"
-#include "gpm-hal-brightness.h"
+#include "gpm-hal-brightness-lcd.h"
+#include "gpm-hal-brightness-kbd.h"
 
 #include "gpm-debug.h"
 #include "gpm-dpms.h"
@@ -58,10 +60,10 @@
 #include "gpm-info.h"
 #include "gpm-graph-widget.h"
 #include "gpm-power.h"
-#include "gpm-feedback-widget.h"
 #include "gpm-tray-icon.h"
 #include "gpm-inhibit.h"
 #include "gpm-polkit.h"
+#include "gpm-button.h"
 #include "gpm-stock-icons.h"
 #include "gpm-manager.h"
 #include "gpm-interface-statistics.h"
@@ -81,18 +83,6 @@ typedef enum {
 	GPM_WARNING_ACTION = 5
 } GpmWarning;
 
-#define GPM_BUTTON_POWER		"power"
-#define GPM_BUTTON_SLEEP		"sleep"
-#define GPM_BUTTON_SUSPEND		"suspend"
-#define GPM_BUTTON_HIBERNATE		"hibernate"
-#define GPM_BUTTON_LID			"lid"
-#define GPM_BUTTON_BRIGHT_UP		"brightness-up"
-#define GPM_BUTTON_BRIGHT_DOWN		"brightness-down"
-#define GPM_BUTTON_BRIGHT_UP_DEP	"brightnessup"	 /* Remove when we depend on HAL 0.5.8 */
-#define GPM_BUTTON_BRIGHT_DOWN_DEP	"brightnessdown" /* as these are the old names */
-#define GPM_BUTTON_LOCK			"lock"
-#define GPM_BUTTON_BATTERY		"battery"
-
 #define GPM_NOTIFY_TIMEOUT_LONG		20	/* seconds */
 #define GPM_NOTIFY_TIMEOUT_SHORT	5	/* seconds */
 
@@ -101,14 +91,15 @@ struct GpmManagerPrivate
 	GConfClient	*gconf_client;
 
 	GpmDpms		*dpms;
+	GpmButton	*button;
 	GpmIdle		*idle;
 	GpmHal		*hal;
 	GpmHalPower	*hal_power;
 	GpmHalCpuFreq	*hal_cpufreq;
 	GpmInfo		*info;
-	GpmFeedback	*feedback;
 	GpmPower	*power;
-	GpmHalBrightness *brightness;
+	GpmHalBrightnessLcd *brightness_lcd;
+	GpmHalBrightnessKbd *brightness_kbd;
 	GpmScreensaver  *screensaver;
 	GpmInhibit	*inhibit;
 	GpmPolkit	*polkit;
@@ -285,11 +276,11 @@ gpm_manager_allowed_suspend (GpmManager *manager,
 
 	*can = FALSE;
 	gconf_ok = gconf_client_get_bool (manager->priv->gconf_client,
-					      GPM_PREF_CAN_SUSPEND, NULL);
+					  GPM_PREF_CAN_SUSPEND, NULL);
 	hal_ok = gpm_hal_power_can_suspend (manager->priv->hal_power);
-#ifdef HAVE_POLKIT
-	polkit_ok = gpm_polkit_is_user_privileged (manager->priv->polkit, "hal-power-suspend");
-#endif
+	if (manager->priv->polkit) {
+		polkit_ok = gpm_polkit_is_user_privileged (manager->priv->polkit, "hal-power-suspend");
+	}
 	if ( gconf_ok && hal_ok && polkit_ok ) {
 		*can = TRUE;
 	}
@@ -319,9 +310,9 @@ gpm_manager_allowed_hibernate (GpmManager *manager,
 	gconf_ok = gconf_client_get_bool (manager->priv->gconf_client,
 					      GPM_PREF_CAN_HIBERNATE, NULL);
 	hal_ok = gpm_hal_power_can_hibernate (manager->priv->hal_power);
-#ifdef HAVE_POLKIT
-	polkit_ok = gpm_polkit_is_user_privileged (manager->priv->polkit, "hal-power-hibernate");
-#endif
+	if (manager->priv->polkit) {
+		polkit_ok = gpm_polkit_is_user_privileged (manager->priv->polkit, "hal-power-hibernate");
+	}
 	if ( gconf_ok && hal_ok && polkit_ok ) {
 		*can = TRUE;
 	}
@@ -343,9 +334,9 @@ gpm_manager_allowed_shutdown (GpmManager *manager,
 	gboolean polkit_ok = TRUE;
 	g_return_val_if_fail (can, FALSE);
 	*can = FALSE;
-#ifdef HAVE_POLKIT
-	polkit_ok = gpm_polkit_is_user_privileged (manager->priv->polkit, "hal-power-shutdown");
-#endif
+	if (manager->priv->polkit) {
+		polkit_ok = gpm_polkit_is_user_privileged (manager->priv->polkit, "hal-power-shutdown");
+	}
 	if ( polkit_ok ) {
 		*can = TRUE;
 	}
@@ -367,9 +358,9 @@ gpm_manager_allowed_reboot (GpmManager *manager,
 	gboolean polkit_ok = TRUE;
 	g_return_val_if_fail (can, FALSE);
 	*can = FALSE;
-#ifdef HAVE_POLKIT
-	polkit_ok = gpm_polkit_is_user_privileged (manager->priv->polkit, "hal-power-reboot");
-#endif
+	if (manager->priv->polkit) {
+		polkit_ok = gpm_polkit_is_user_privileged (manager->priv->polkit, "hal-power-reboot");
+	}
 	if ( polkit_ok ) {
 		*can = TRUE;
 	}
@@ -486,14 +477,14 @@ get_stock_id (GpmManager *manager,
 }
 
 /**
- * tray_icon_update:
+ * gpm_manager_sync_tray_icon:
  * @manager: This manager class instance
  *
  * Update the tray icon and set the correct tooltip when required, or remove
  * (hide) the icon when no longer required by policy.
  **/
 static void
-tray_icon_update (GpmManager *manager)
+gpm_manager_sync_tray_icon (GpmManager *manager)
 {
 	gchar *stock_id = NULL;
 	gchar *icon_policy_str;
@@ -516,10 +507,8 @@ tray_icon_update (GpmManager *manager)
 
 		gpm_tray_icon_set_image_from_stock (GPM_TRAY_ICON (manager->priv->tray_icon),
 						    stock_id);
-
 		/* make sure that we are visible */
 		gpm_tray_icon_show (GPM_TRAY_ICON (manager->priv->tray_icon), TRUE);
-
 		g_free (stock_id);
 
 		gpm_power_get_status_summary (manager->priv->power, &tooltip, NULL);
@@ -537,13 +526,13 @@ tray_icon_update (GpmManager *manager)
 }
 
 /**
- * sync_dpms_policy:
+ * gpm_manager_sync_policy_dpms:
  * @manager: This manager class instance
  *
  * Sync the DPMS policy with what we have set in gconf.
  **/
 static void
-sync_dpms_policy (GpmManager *manager)
+gpm_manager_sync_policy_dpms (GpmManager *manager)
 {
 	GError  *error;
 	gboolean res;
@@ -602,7 +591,7 @@ sync_dpms_policy (GpmManager *manager)
 
 	/* check if method is valid */
 	if (method == GPM_DPMS_METHOD_UNKNOWN) {
-		g_warning ("DPMS method unknown. Possible schema problem!");
+		gpm_warning ("DPMS method unknown. Possible schema problem!");
 		return;
 	}
 
@@ -642,7 +631,7 @@ sync_dpms_policy (GpmManager *manager)
 		gpm_warning ("unknown dpms mode!");
 	}
 
-	gpm_debug ("DPMS parameters %d %d %d, method '%i'\n", standby, suspend, off, method);
+	gpm_debug ("DPMS parameters %d %d %d, method '%i'", standby, suspend, off, method);
 
 	error = NULL;
 	res = gpm_dpms_set_enabled (manager->priv->dpms, TRUE, &error);
@@ -709,7 +698,94 @@ update_lid_throttle (GpmManager	*manager,
 }
 
 /**
- * change_power_policy:
+ * gpm_manager_sync_policy_cpufreq:
+ * @manager: This manager class instance
+ * @on_ac: If we are on AC power
+ *
+ * Changes the cpufreq policy if required
+ **/
+static gboolean
+gpm_manager_sync_policy_cpufreq (GpmManager *manager)
+{
+	gboolean     cpufreq_consider_nice;
+	gboolean     on_ac;
+	guint	     cpufreq_performance;
+	gchar       *cpufreq_policy;
+	GConfClient *client;
+	GpmHalCpuFreqEnum cpufreq_type;
+
+	if (manager->priv->hal_cpufreq == NULL) {
+		gpm_debug ("cpufreq support absent, so ignoring");
+		return FALSE;
+	}
+
+	gpm_power_get_on_ac (manager->priv->power, &on_ac, NULL);
+
+	client = manager->priv->gconf_client;
+
+	if (on_ac) {
+		cpufreq_consider_nice = gconf_client_get_bool (client, GPM_PREF_USE_NICE, NULL);
+		cpufreq_policy = gconf_client_get_string (client, GPM_PREF_AC_CPUFREQ_POLICY, NULL);
+		cpufreq_performance = gconf_client_get_int (client, GPM_PREF_AC_CPUFREQ_VALUE, NULL);
+	} else {
+		cpufreq_consider_nice = gconf_client_get_bool (client, GPM_PREF_USE_NICE, NULL);
+		cpufreq_policy = gconf_client_get_string (client, GPM_PREF_BATTERY_CPUFREQ_POLICY, NULL);
+		cpufreq_performance = gconf_client_get_int (client, GPM_PREF_BATTERY_CPUFREQ_VALUE, NULL);
+	}
+
+	/* use enumerated value */
+	cpufreq_type = gpm_hal_cpufreq_string_to_enum (cpufreq_policy);
+	g_free (cpufreq_policy);
+
+	/* change to the right governer and settings */
+	gpm_hal_cpufreq_set_consider_nice (manager->priv->hal_cpufreq, cpufreq_consider_nice);
+	gpm_hal_cpufreq_set_governor (manager->priv->hal_cpufreq, cpufreq_type);
+	gpm_hal_cpufreq_set_performance (manager->priv->hal_cpufreq, cpufreq_performance);
+	return TRUE;
+}
+
+/**
+ * gpm_manager_sync_policy_brightness:
+ * @manager: This manager class instance
+ * @on_ac: If we are on AC power
+ *
+ * Changes the policy setting brightness
+ **/
+static void
+gpm_manager_sync_policy_brightness (GpmManager *manager)
+{
+	gboolean     on_ac;
+	gboolean     do_laptop_lcd;
+	guint	     brightness_lcd;
+	guint	     brightness_kbd;
+	GConfClient *client;
+
+	client = manager->priv->gconf_client;
+
+	gpm_power_get_on_ac (manager->priv->power, &on_ac, NULL);
+
+	if (on_ac) {
+		brightness_lcd = gconf_client_get_int (client, GPM_PREF_AC_BRIGHTNESS, NULL);
+		brightness_kbd = gconf_client_get_int (client, GPM_PREF_AC_BRIGHTNESS_KBD, NULL);
+	} else {
+		brightness_lcd = gconf_client_get_int (client, GPM_PREF_BATTERY_BRIGHTNESS, NULL);
+		brightness_kbd = gconf_client_get_int (client, GPM_PREF_BATTERY_BRIGHTNESS_KBD, NULL);
+	}
+
+	/* only do brightness changes if we have the hardware */
+	do_laptop_lcd = gconf_client_get_bool (manager->priv->gconf_client,
+					       GPM_PREF_DISPLAY_STATE_CHANGE, NULL);
+	if (do_laptop_lcd && manager->priv->brightness_lcd) {
+		gpm_hal_brightness_lcd_set_std (manager->priv->brightness_lcd, brightness_lcd);
+	}
+
+	if (manager->priv->brightness_kbd) {
+		gpm_hal_brightness_kbd_set_std (manager->priv->brightness_kbd, brightness_kbd);
+	}
+}
+
+/**
+ * gpm_manager_sync_policy_sleep:
  * @manager: This manager class instance
  * @on_ac: If we are on AC power
  *
@@ -719,62 +795,35 @@ update_lid_throttle (GpmManager	*manager,
  * monitor DPMS instead when on batteries to save power.
  **/
 static void
-change_power_policy (GpmManager *manager,
-		     gboolean	 on_ac)
+gpm_manager_sync_policy_sleep (GpmManager *manager)
 {
-	guint	     brightness;
 	guint	     sleep_display;
 	guint	     sleep_computer;
+	gboolean     on_ac;
 	gboolean     power_save;
-	gboolean     cpufreq_consider_nice;
-	gboolean     do_laptop_lcd;
-	guint	     cpufreq_performance;
-	gchar       *cpufreq_policy;
 	GConfClient *client;
-	GpmHalCpuFreqEnum cpufreq_type;
 
 	client = manager->priv->gconf_client;
 
+	gpm_power_get_on_ac (manager->priv->power, &on_ac, NULL);
+
 	if (on_ac) {
-		brightness = gconf_client_get_int (client, GPM_PREF_AC_BRIGHTNESS, NULL);
 		sleep_computer = gconf_client_get_int (client, GPM_PREF_AC_SLEEP_COMPUTER, NULL);
 		sleep_display = gconf_client_get_int (client, GPM_PREF_AC_SLEEP_DISPLAY, NULL);
 		power_save = gconf_client_get_bool (client, GPM_PREF_AC_LOWPOWER, NULL);
-		cpufreq_consider_nice = gconf_client_get_bool (client, GPM_PREF_USE_NICE, NULL);
-		cpufreq_policy = gconf_client_get_string (client, GPM_PREF_AC_CPUFREQ_POLICY, NULL);
-		cpufreq_performance = gconf_client_get_int (client, GPM_PREF_AC_CPUFREQ_VALUE, NULL);
 	} else {
-		brightness = gconf_client_get_int (client, GPM_PREF_BATTERY_BRIGHTNESS, NULL);
 		sleep_computer = gconf_client_get_int (client, GPM_PREF_BATTERY_SLEEP_COMPUTER, NULL);
 		sleep_display = gconf_client_get_int (client, GPM_PREF_BATTERY_SLEEP_DISPLAY, NULL);
 		/* todo: what about when on UPS? */
 		power_save = gconf_client_get_bool (client, GPM_PREF_BATTERY_LOWPOWER, NULL);
-		cpufreq_consider_nice = gconf_client_get_bool (client, GPM_PREF_USE_NICE, NULL);
-		cpufreq_policy = gconf_client_get_string (client, GPM_PREF_BATTERY_CPUFREQ_POLICY, NULL);
-		cpufreq_performance = gconf_client_get_int (client, GPM_PREF_BATTERY_CPUFREQ_VALUE, NULL);
 	}
 
 	/* only do brightness changes if we have the hardware */
-	do_laptop_lcd = gconf_client_get_bool (manager->priv->gconf_client,
-					       GPM_PREF_DISPLAY_STATE_CHANGE, NULL);
-	if (do_laptop_lcd && gpm_hal_brightness_has_hardware (manager->priv->brightness)) {
-		gpm_hal_brightness_set_level_std (manager->priv->brightness, brightness);
-		gpm_hal_brightness_set (manager->priv->brightness);
-	}
-
-	/* change to the right governer and settings */
-	cpufreq_type = gpm_hal_cpufreq_string_to_enum (cpufreq_policy);
-	gpm_hal_cpufreq_set_consider_nice (manager->priv->hal_cpufreq, cpufreq_consider_nice);
-	gpm_hal_cpufreq_set_governor (manager->priv->hal_cpufreq, cpufreq_type);
-	gpm_hal_cpufreq_set_performance (manager->priv->hal_cpufreq, cpufreq_performance);
-	g_free (cpufreq_policy);
-
 	gpm_hal_power_enable_power_save (manager->priv->hal_power, power_save);
 	update_ac_throttle (manager, on_ac);
 
 	/* set the new sleep (inactivity) value */
 	gpm_idle_set_system_timeout (manager->priv->idle, sleep_computer);
-	sync_dpms_policy (manager);
 }
 
 /**
@@ -830,8 +879,6 @@ gpm_manager_blank_screen (GpmManager *manager,
 	gboolean ret = TRUE;
 	GError  *error = NULL;
 
-	gpm_info_event_log (manager->priv->info, GPM_GRAPH_WIDGET_EVENT_DPMS_OFF, NULL);
-
 	do_lock = gpm_manager_get_lock_policy (manager,
 					       GPM_PREF_LOCK_ON_BLANK_SCREEN);
 	if (do_lock) {
@@ -945,8 +992,6 @@ manager_policy_do (GpmManager  *manager,
 		gpm_manager_hibernate (manager, NULL);
 
 	} else if (strcmp (action, ACTION_BLANK) == 0) {
-		manager_explain_reason (manager, GPM_GRAPH_WIDGET_EVENT_DPMS_OFF,
-					_("DPMS blanking screen"), reason);
 		gpm_manager_blank_screen (manager, NULL);
 
 	} else if (strcmp (action, ACTION_SHUTDOWN) == 0) {
@@ -1284,7 +1329,7 @@ gpm_manager_hibernate (GpmManager *manager,
 		gpm_networkmanager_wake ();
 	}
 
-	sync_dpms_policy (manager);
+	gpm_manager_sync_policy_dpms (manager);
 
 	/* save the time that we resumed */
 	gpm_manager_reset_event_time (manager);
@@ -1310,10 +1355,6 @@ gpm_manager_suspend (GpmManager *manager,
 	gboolean ret;
 	gboolean do_lock;
 	gboolean nm_sleep;
-	GpmPowerStatus status;
-	guint charge_before_suspend;
-	gint charge_difference;
-	gchar *message;
 
 	gpm_manager_allowed_suspend (manager, &allowed, NULL);
 
@@ -1338,14 +1379,6 @@ gpm_manager_suspend (GpmManager *manager,
 		gpm_networkmanager_sleep ();
 	}
 
-
-	/* We save the current charge in mWh so we can see how much power we
-	   lost or gained over the suspend cycle */
-	gpm_power_get_battery_status (manager->priv->power,
-				      GPM_POWER_KIND_PRIMARY,
-				      &status);
-	charge_before_suspend = status.current_charge;
-
 	/* Do the suspend */
 	ret = gpm_hal_power_suspend (manager->priv->hal_power, 0);
 	manager_explain_reason (manager, GPM_GRAPH_WIDGET_EVENT_RESUME,
@@ -1354,25 +1387,6 @@ gpm_manager_suspend (GpmManager *manager,
 	/* We need to refresh all the power caches */
 	gpm_power_update_all (manager->priv->power);
 
-	/* Get the difference in charge and add it to the event log */
-	gpm_power_get_battery_status (manager->priv->power,
-				      GPM_POWER_KIND_PRIMARY,
-				      &status);
-	charge_difference = status.current_charge - charge_before_suspend;
-	if (charge_difference != 0) {
-		if (charge_difference > 0) {
-			message = g_strdup_printf (_("Battery charged %imWh during suspend"),
-						   charge_difference);
-		} else {
-			message = g_strdup_printf (_("Battery discharged %imWh during suspend"),
-						   -charge_difference);
-		}
-		gpm_info_event_log (manager->priv->info,
-				    GPM_GRAPH_WIDGET_EVENT_NOTIFICATION,
-				    message);
-		g_free (message);
-	}
-
 	if (! ret) {
 		gboolean show_notify;
 
@@ -1385,6 +1399,7 @@ gpm_manager_suspend (GpmManager *manager,
 						     GPM_PREF_NOTIFY_HAL_ERROR, NULL);
 		if (show_notify) {
 			const gchar *title;
+			gchar *message;
 			message = g_strdup_printf (_("Your computer failed to %s.\n"
 						     "Check the help file for common problems."),
 						     _("suspend"));
@@ -1412,7 +1427,7 @@ gpm_manager_suspend (GpmManager *manager,
 		gpm_networkmanager_wake ();
 	}
 
-	sync_dpms_policy (manager);
+	gpm_manager_sync_policy_dpms (manager);
 
 	/* save the time that we resumed */
 	gpm_manager_reset_event_time (manager);
@@ -1526,16 +1541,21 @@ idle_changed_cb (GpmIdle    *idle,
 		/* Should we resume the screen? */
 		do_laptop_dim = gconf_client_get_bool (manager->priv->gconf_client,
 						       GPM_PREF_DISPLAY_IDLE_DIM, NULL);
-		if (do_laptop_dim && gpm_hal_brightness_has_hardware (manager->priv->brightness)) {
+		if (do_laptop_dim && manager->priv->brightness_lcd) {
 			/* resume to the previous brightness */
 			manager_explain_reason (manager, GPM_GRAPH_WIDGET_EVENT_SCREEN_RESUME,
 						_("Screen resume"),
 						_("idle mode ended"));
-			gpm_hal_brightness_undim (manager->priv->brightness);
+			gpm_hal_brightness_lcd_undim (manager->priv->brightness_lcd);
+		}
+
+		/* turn keyboard backlight back on */
+		if (manager->priv->brightness_kbd) {
+			gpm_hal_brightness_kbd_undim (manager->priv->brightness_kbd);
 		}
 
 		/* sync timeouts */
-		sync_dpms_policy (manager);
+		gpm_manager_sync_policy_dpms (manager);
 
 		break;
 	case GPM_IDLE_MODE_SESSION:
@@ -1552,16 +1572,21 @@ idle_changed_cb (GpmIdle    *idle,
 		/* Should we dim the screen? */
 		do_laptop_dim = gconf_client_get_bool (manager->priv->gconf_client,
 						       GPM_PREF_DISPLAY_IDLE_DIM, NULL);
-		if (do_laptop_dim && gpm_hal_brightness_has_hardware (manager->priv->brightness)) {
+		if (do_laptop_dim && manager->priv->brightness_lcd) {
 			/* Dim the screen, fixes #328564 */
 			manager_explain_reason (manager, GPM_GRAPH_WIDGET_EVENT_SCREEN_DIM,
 						_("Screen dim"),
 						_("idle mode started"));
-			gpm_hal_brightness_dim (manager->priv->brightness);
+			gpm_hal_brightness_lcd_dim (manager->priv->brightness_lcd);
+		}
+
+		/* dim keyboard backlight */
+		if (do_laptop_dim && manager->priv->brightness_kbd) {
+			gpm_hal_brightness_kbd_dim (manager->priv->brightness_kbd);
 		}
 
 		/* sync timeouts */
-		sync_dpms_policy (manager);
+		gpm_manager_sync_policy_dpms (manager);
 
 		break;
 	case GPM_IDLE_MODE_SYSTEM:
@@ -1573,8 +1598,12 @@ idle_changed_cb (GpmIdle    *idle,
 		if (! gpm_manager_is_inhibit_valid (manager, "timeout action")) {
 			return;
 		}
-		/* can only be hibernate or suspend */
-		manager_policy_do (manager, GPM_PREF_SLEEP_TYPE, _("the system state is idle"));
+		/* can only be hibernate, suspend or nothing */
+		if (on_ac) {
+			manager_policy_do (manager, GPM_PREF_AC_SLEEP_TYPE, _("the system state is idle"));
+		} else {
+			manager_policy_do (manager, GPM_PREF_BATTERY_SLEEP_TYPE, _("the system state is idle"));
+		}
 
 		break;
 	default:
@@ -1746,30 +1775,10 @@ lid_button_pressed (GpmManager *manager,
 		}
 	} else {
 		/* we turn the lid dpms back on unconditionally */
-		manager_explain_reason (manager, GPM_GRAPH_WIDGET_EVENT_DPMS_ON,
-					_("Turning LCD panel back on"),
-					_("laptop lid re-opened"));
 		gpm_manager_unblank_screen (manager, NULL);
 	}
 }
 
-/**
- * brightness_step_changed_cb:
- * @brightness: The power class instance
- * @percentage: The new brightness setting
- * @manager: This manager class instance
- *
- * The callback when the brightness is stepped up or stepped down
- **/
-static void
-brightness_step_changed_cb (GpmHalBrightness *brightness,
-			    int		      percentage,
-			    GpmManager	     *manager)
-{
-	gpm_debug ("Need to diplay feedback value %i", percentage);
-	gpm_feedback_display_value (manager->priv->feedback, (float) percentage / 100.0f);
-}
-
 /**
  * power_button_pressed_cb:
  * @power: The power class instance
@@ -1787,9 +1796,6 @@ power_button_pressed_cb (GpmPower    *power,
 {
 	gpm_debug ("Button press event type=%s state=%d", type, state);
 
-	/* simulate user input */
-	gpm_screensaver_poke (manager->priv->screensaver);
-
 	if (strcmp (type, GPM_BUTTON_POWER) == 0) {
 		power_button_pressed (manager);
 
@@ -1807,11 +1813,30 @@ power_button_pressed_cb (GpmPower    *power,
 
 	} else if ((strcmp (type, GPM_BUTTON_BRIGHT_UP) == 0) ||
 		   (strcmp (type, GPM_BUTTON_BRIGHT_UP_DEP) == 0)) {
-		gpm_hal_brightness_up (manager->priv->brightness);
+		if (manager->priv->brightness_lcd) {
+			gpm_hal_brightness_lcd_up (manager->priv->brightness_lcd);
+		}
 
 	} else if ((strcmp (type, GPM_BUTTON_BRIGHT_DOWN) == 0) ||
 		   (strcmp (type, GPM_BUTTON_BRIGHT_DOWN_DEP) == 0)) {
-		gpm_hal_brightness_down (manager->priv->brightness);
+		if (manager->priv->brightness_lcd) {
+			gpm_hal_brightness_lcd_down (manager->priv->brightness_lcd);
+		}
+
+	} else if (strcmp (type, GPM_BUTTON_KBD_BRIGHT_UP) == 0) {
+		if (manager->priv->brightness_kbd) {
+			gpm_hal_brightness_kbd_up (manager->priv->brightness_kbd);
+		}
+
+	} else if (strcmp (type, GPM_BUTTON_KBD_BRIGHT_DOWN) == 0) {
+		if (manager->priv->brightness_kbd) {
+			gpm_hal_brightness_kbd_down (manager->priv->brightness_kbd);
+		}
+
+	} else if (strcmp (type, GPM_BUTTON_KBD_BRIGHT_TOGGLE) == 0) {
+		if (manager->priv->brightness_kbd) {
+			gpm_hal_brightness_kbd_toggle (manager->priv->brightness_kbd);
+		}
 
 	} else if (strcmp (type, GPM_BUTTON_LOCK) == 0) {
 		gpm_screensaver_lock (manager->priv->screensaver);
@@ -1851,17 +1876,22 @@ power_on_ac_changed_cb (GpmPower   *power,
 		manager->priv->last_primary_warning = GPM_WARNING_NONE;
 	}
 
-	tray_icon_update (manager);
+	gpm_manager_sync_tray_icon (manager);
 
 	if (on_ac) {
 		/* for where we add back the ac_adapter before
 		 * the "AC Power unplugged" message times out. */
 		gpm_tray_icon_cancel_notify (GPM_TRAY_ICON (manager->priv->tray_icon));
 	}
-	change_power_policy (manager, on_ac);
+
+	gpm_manager_sync_policy_sleep (manager);
+	gpm_manager_sync_policy_cpufreq (manager);
+	gpm_manager_sync_policy_brightness (manager);
+	gpm_manager_sync_policy_dpms (manager);
 
 	gpm_debug ("emitting on-ac-changed : %i", on_ac);
 	g_signal_emit (manager, signals [ON_AC_CHANGED], 0, on_ac);
+
 	if (on_ac) {
 		gpm_info_event_log (manager->priv->info,
 				    GPM_GRAPH_WIDGET_EVENT_ON_AC,
@@ -2079,6 +2109,12 @@ battery_status_changed_primary (GpmManager     *manager,
 		return;
 	}
 
+	/* Wait until data is trusted... */
+	if (gpm_power_get_data_is_trusted (manager->priv->power) == FALSE) {
+		gpm_debug ("Data is not yet trusted.. wait..");
+		return;
+	}
+
 	/* As the level is more critical than the last warning, save it */
 	manager->priv->last_primary_warning = warning_type;
 
@@ -2369,7 +2405,7 @@ power_battery_status_changed_cb (GpmPower    *power,
 {
 	GpmPowerStatus battery_status;
 
-	tray_icon_update (manager);
+	gpm_manager_sync_tray_icon (manager);
 
 	gpm_power_get_battery_status (manager->priv->power, battery_kind, &battery_status);
 
@@ -2392,6 +2428,49 @@ power_battery_status_changed_cb (GpmPower    *power,
 	}
 }
 
+/**
+ * power_battery_status_perhaps_recall_cb:
+ * @power: The power class instance
+ * @vendor: The battery vendor, e.g. "DELL"
+ * @manager: This manager class instance
+ *
+ * This function splits up the battery status changed callback, and calls
+ * different functions for each of the device types.
+ **/
+static void
+power_battery_status_perhaps_recall_cb (GpmPower    *power,
+				       const gchar *oem_vendor,
+				       const gchar *website,
+				       GpmManager  *manager)
+{
+	gchar *msg;
+
+	/* check to see if HAL has given us all the right info */
+	if (oem_vendor == NULL || website == NULL) {
+		gpm_warning ("Possibly a potential critical hardware problem, "
+			     "but not enough data from HAL to report to the user");
+		return;
+	}
+
+	const gchar *title = _("Your battery may have been recalled");
+	const char *problem = _("The battery in your computer may have been "
+				"recalled by the manufacturer and you may be "
+				"at risk.\n");
+	const char *action = _("For more information visit the following web site:\n");
+
+	msg = g_strdup_printf ("%s\n%s\n<a href=\"%s\">%s</a>",
+			       problem, action, website, oem_vendor);
+				 
+/* TODO:
+ [x] do not notify me of this anymore.
+ */
+	gpm_tray_icon_notify (GPM_TRAY_ICON (manager->priv->tray_icon),
+			      title, msg, 0,
+			      GTK_STOCK_DIALOG_WARNING,
+			      GPM_NOTIFY_URGENCY_NORMAL);
+	g_free (msg);
+}
+
 /**
  * gpm_manager_class_init:
  * @klass: The GpmManagerClass
@@ -2440,7 +2519,6 @@ gconf_key_changed_cb (GConfClient *client,
 		      GConfEntry  *entry,
 		      gpointer	   user_data)
 {
-	gint	    value = 0;
 	gint	    brightness;
 	GpmManager *manager = GPM_MANAGER (user_data);
 	gboolean    on_ac;
@@ -2457,57 +2535,35 @@ gconf_key_changed_cb (GConfClient *client,
 
 	if (strcmp (entry->key, GPM_PREF_ICON_POLICY) == 0) {
 
-		tray_icon_update (manager);
+		gpm_manager_sync_tray_icon (manager);
 
-	} else if (strcmp (entry->key, GPM_PREF_BATTERY_SLEEP_COMPUTER) == 0) {
-		/* set new suspend timeouts */
-		value = gconf_client_get_int (client, entry->key, NULL);
+	} else if (strcmp (entry->key, GPM_PREF_AC_CPUFREQ_POLICY) == 0 ||
+		   strcmp (entry->key, GPM_PREF_AC_CPUFREQ_VALUE) == 0 ||
+		   strcmp (entry->key, GPM_PREF_BATTERY_CPUFREQ_POLICY) == 0 ||
+		   strcmp (entry->key, GPM_PREF_BATTERY_CPUFREQ_VALUE) == 0 ||
+		   strcmp (entry->key, GPM_PREF_USE_NICE) == 0) {
 
-		if (! on_ac) {
-			gpm_idle_set_system_timeout (manager->priv->idle, value);
-		}
+		gpm_manager_sync_policy_cpufreq (manager);
 
-	} else if (strcmp (entry->key, GPM_PREF_AC_SLEEP_COMPUTER) == 0) {
+	} else if (strcmp (entry->key, GPM_PREF_BATTERY_SLEEP_COMPUTER) == 0 ||
+		   strcmp (entry->key, GPM_PREF_AC_SLEEP_COMPUTER) == 0) {
 
-		/* set new suspend timeouts */
-		value = gconf_client_get_int (client, entry->key, NULL);
+		gpm_manager_sync_policy_sleep (manager);
 
-		if (on_ac) {
-			gpm_idle_set_system_timeout (manager->priv->idle,
-						     value);
-		}
+	} else if (strcmp (entry->key, GPM_PREF_BATTERY_SLEEP_DISPLAY) == 0 ||
+		   strcmp (entry->key, GPM_PREF_AC_SLEEP_DISPLAY) == 0 ||
+		   strcmp (entry->key, GPM_PREF_AC_DPMS_METHOD) == 0 ||
+		   strcmp (entry->key, GPM_PREF_BATTERY_DPMS_METHOD) == 0) {
 
-	} else if (strcmp (entry->key, GPM_PREF_BATTERY_SLEEP_DISPLAY) == 0) {
+		gpm_manager_sync_policy_dpms (manager);
 
-		sync_dpms_policy (manager);
+	} else if (strcmp (entry->key, GPM_PREF_AC_BRIGHTNESS) == 0 ||
+		   strcmp (entry->key, GPM_PREF_BATTERY_BRIGHTNESS) == 0 ||
+		   strcmp (entry->key, GPM_PREF_AC_BRIGHTNESS_KBD) == 0 ||
+		   strcmp (entry->key, GPM_PREF_BATTERY_BRIGHTNESS_KBD) == 0) {
 
-	} else if (strcmp (entry->key, GPM_PREF_AC_SLEEP_DISPLAY) == 0) {
+		gpm_manager_sync_policy_brightness (manager);
 
-		sync_dpms_policy (manager);
-
-	} else if (strcmp (entry->key, GPM_PREF_AC_DPMS_METHOD) == 0) {
-
-		sync_dpms_policy (manager);
-
-	} else if (strcmp (entry->key, GPM_PREF_BATTERY_DPMS_METHOD) == 0) {
-
-		sync_dpms_policy (manager);
-
-	} else if (strcmp (entry->key, GPM_PREF_AC_BRIGHTNESS) == 0) {
-
-		if (on_ac) {
-			brightness = gconf_client_get_int (client, GPM_PREF_AC_BRIGHTNESS, NULL);
-			gpm_hal_brightness_set_level_std (manager->priv->brightness, brightness);
-			gpm_hal_brightness_set (manager->priv->brightness);
-		}
-
-	} else if (strcmp (entry->key, GPM_PREF_BATTERY_BRIGHTNESS) == 0) {
-
-		if (! on_ac) {
-			brightness = gconf_client_get_int (client, GPM_PREF_BATTERY_BRIGHTNESS, NULL);
-			gpm_hal_brightness_set_level_std (manager->priv->brightness, brightness);
-			gpm_hal_brightness_set (manager->priv->brightness);
-		}
 
 	} else if (strcmp (entry->key, GPM_PREF_CAN_SUSPEND) == 0) {
 		gpm_manager_allowed_suspend (manager, &enabled, NULL);
@@ -2541,7 +2597,9 @@ gconf_key_changed_cb (GConfClient *client,
 	} else if (strcmp (entry->key, GPM_PREF_PANEL_DIM_BRIGHTNESS) == 0) {
 		brightness = gconf_client_get_int (manager->priv->gconf_client,
 						   GPM_PREF_PANEL_DIM_BRIGHTNESS, NULL);
-		gpm_hal_brightness_set_level_dim (manager->priv->brightness, brightness);
+		if (manager->priv->brightness_lcd) {
+			gpm_hal_brightness_lcd_set_dim (manager->priv->brightness_lcd, brightness);
+		}
 
 	} else if (strcmp (entry->key, GPM_PREF_ENABLE_BEEPING) == 0) {
 		manager->priv->enable_beeping =
@@ -2618,31 +2676,7 @@ hal_battery_removed_cb (GpmHalMonitor *monitor,
 			GpmManager    *manager)
 {
 	gpm_debug ("Battery Removed: %s", udi);
-	tray_icon_update (manager);
-}
-
-/**
- * gpm_manager_tray_icon_show_info:
- * @manager: This manager class instance
- * @tray: The tray object
- **/
-static void
-gpm_manager_tray_icon_show_info (GpmManager   *manager,
-				 GpmTrayIcon  *tray)
-{
-	gpm_debug ("Received show-info signal from tray icon");
-	gpm_info_show_window (manager->priv->info);
-}
-
-/**
- * tray_icon_destroyed:
- **/
-static void
-tray_icon_destroyed (GtkObject *object, gpointer user_data)
-{
-	GpmManager *manager = user_data;
-
-	manager->priv->tray_icon = gpm_tray_icon_new ();
+	gpm_manager_sync_tray_icon (manager);
 }
 
 /**
@@ -2667,9 +2701,9 @@ screensaver_auth_request_cb (GpmScreensaver *screensaver,
 
 	/* TODO: This may be a bid of a bodge, as we will have multiple
 		 resume requests -- maybe this need a logic cleanup */
-	if (gpm_hal_brightness_has_hardware (manager->priv->brightness)) {
+	if (manager->priv->brightness_lcd) {
 		gpm_debug ("undimming lcd due to auth begin");
-		gpm_hal_brightness_undim (manager->priv->brightness);
+		gpm_hal_brightness_lcd_undim (manager->priv->brightness_lcd);
 	}
 
 	/* We turn on the monitor unconditionally, as we may be using
@@ -2714,7 +2748,7 @@ static void
 hal_daemon_monitor_cb (GpmHal     *hal,
 		     GpmManager *manager)
 {
-	tray_icon_update (manager);
+	gpm_manager_sync_tray_icon (manager);
 }
 
 /**
@@ -2744,6 +2778,17 @@ gpm_manager_init (GpmManager *manager)
 			  G_CALLBACK (power_on_ac_changed_cb), manager);
 	g_signal_connect (manager->priv->power, "battery-status-changed",
 			  G_CALLBACK (power_battery_status_changed_cb), manager);
+	g_signal_connect (manager->priv->power, "battery-perhaps-recall",
+			  G_CALLBACK (power_battery_status_perhaps_recall_cb), manager);
+
+	manager->priv->button = gpm_button_new ();
+	if (manager->priv->button) {
+		g_signal_connect (manager->priv->button, "button-pressed",
+				  G_CALLBACK (power_button_pressed_cb), manager);
+	} else {
+		gpm_warning ("You have no XEVENTS support. You may not get "
+			     "buttons to work with very new kernels");
+	}
 
 	manager->priv->hal = gpm_hal_new ();
 	g_signal_connect (manager->priv->hal, "daemon-start",
@@ -2779,11 +2824,8 @@ gpm_manager_init (GpmManager *manager)
 				 NULL,
 				 NULL);
 
-	manager->priv->brightness = gpm_hal_brightness_new ();
-	g_signal_connect (manager->priv->brightness, "brightness-step-changed",
-			  G_CALLBACK (brightness_step_changed_cb), manager);
-
-	manager->priv->feedback = gpm_feedback_new ();
+	manager->priv->brightness_lcd = gpm_hal_brightness_lcd_new ();
+	manager->priv->brightness_kbd = gpm_hal_brightness_kbd_new ();
 
 	manager->priv->idle = gpm_idle_new ();
 	g_signal_connect (manager->priv->idle, "changed",
@@ -2800,14 +2842,11 @@ gpm_manager_init (GpmManager *manager)
 	/* use a class to handle the complex stuff */
 	manager->priv->inhibit = gpm_inhibit_new ();
 
-#ifdef HAVE_POLKIT
+	/* this will be NULL if we don't compile in support */
 	manager->priv->polkit = gpm_polkit_new ();
-#endif
 
 	gpm_debug ("creating new tray icon");
 	manager->priv->tray_icon = gpm_tray_icon_new ();
-	g_signal_connect (G_OBJECT (manager->priv->tray_icon), "destroy",
-			  G_CALLBACK (tray_icon_destroyed), manager);
 
 	gpm_debug ("initialising info infrastructure");
 	manager->priv->info = gpm_info_new ();
@@ -2815,7 +2854,8 @@ gpm_manager_init (GpmManager *manager)
 	/* add the new statistics DBUS interface */
 	connection = dbus_g_bus_get (DBUS_BUS_SESSION, &error);
 	dbus_g_object_type_install_info (GPM_TYPE_INFO, &dbus_glib_gpm_statistics_object_info);
-	dbus_g_connection_register_g_object (connection, "/org/gnome/PowerManager/Statistics", G_OBJECT (manager->priv->info));
+	dbus_g_connection_register_g_object (connection, "/org/gnome/PowerManager/Statistics",
+					     G_OBJECT (manager->priv->info));
 
 	/* only show the suspend and hibernate icons if we can do the action,
 	   and the policy allows the actions in the menu */
@@ -2838,17 +2878,16 @@ gpm_manager_init (GpmManager *manager)
 				 G_CALLBACK (gpm_manager_tray_icon_hibernate),
 				 manager,
 				 G_CONNECT_SWAPPED);
-	g_signal_connect_object (G_OBJECT (manager->priv->tray_icon),
-				 "show-info",
-				 G_CALLBACK (gpm_manager_tray_icon_show_info),
-				 manager,
-				 G_CONNECT_SWAPPED);
 
 	/* coldplug so we are in the correct state at startup */
-	sync_dpms_policy (manager);
+	gpm_manager_sync_policy_dpms (manager);
 	gpm_power_get_on_ac (manager->priv->power, &on_ac, NULL);
-	change_power_policy (manager, on_ac);
-	tray_icon_update (manager);
+
+	gpm_manager_sync_policy_sleep (manager);
+	gpm_manager_sync_policy_cpufreq (manager);
+	gpm_manager_sync_policy_brightness (manager);
+	gpm_manager_sync_policy_dpms (manager);
+	gpm_manager_sync_tray_icon (manager);
 
 	g_signal_connect (manager->priv->dpms, "mode-changed",
 			  G_CALLBACK (dpms_mode_changed_cb), manager);
@@ -2918,11 +2957,11 @@ gpm_manager_init (GpmManager *manager)
 							   GPM_PREF_ACTION_TIME, NULL);
 
 	/* Get dim settings */
-	if (gpm_hal_brightness_has_hardware (manager->priv->brightness)) {
+	if (manager->priv->brightness_lcd) {
 		lcd_dim_brightness = gconf_client_get_int (manager->priv->gconf_client,
 							   GPM_PREF_PANEL_DIM_BRIGHTNESS, NULL);
 		gpm_debug ("lcd_dim_brightness is %i", lcd_dim_brightness);
-		gpm_hal_brightness_set_level_dim (manager->priv->brightness, lcd_dim_brightness);
+		gpm_hal_brightness_lcd_set_dim (manager->priv->brightness_lcd, lcd_dim_brightness);
 	}
 
 	/* Do we beep? */
@@ -2952,23 +2991,35 @@ gpm_manager_finalize (GObject *object)
 
 	g_return_if_fail (manager->priv != NULL);
 
+	/* compulsory gobjects */
 	g_object_unref (manager->priv->gconf_client);
 	g_object_unref (manager->priv->hal);
 	g_object_unref (manager->priv->hal_power);
-	g_object_unref (manager->priv->hal_cpufreq);
 	g_object_unref (manager->priv->dpms);
 	g_object_unref (manager->priv->idle);
 	g_object_unref (manager->priv->info);
-#ifdef HAVE_POLKIT
-	g_object_unref (manager->priv->polkit);
-#endif
 	g_object_unref (manager->priv->power);
-	g_object_unref (manager->priv->brightness);
 	g_object_unref (manager->priv->tray_icon);
-	g_object_unref (manager->priv->feedback);
 	g_object_unref (manager->priv->inhibit);
 	g_object_unref (manager->priv->screensaver);
 
+	/* optional gobjects */
+	if (manager->priv->button) {
+		g_object_unref (manager->priv->button);
+	}
+	if (manager->priv->hal_cpufreq) {
+		g_object_unref (manager->priv->hal_cpufreq);
+	}
+	if (manager->priv->polkit) {
+		g_object_unref (manager->priv->polkit);
+	}
+	if (manager->priv->brightness_lcd) {
+		g_object_unref (manager->priv->brightness_lcd);
+	}
+	if (manager->priv->brightness_kbd) {
+		g_object_unref (manager->priv->brightness_kbd);
+	}
+
 	G_OBJECT_CLASS (gpm_manager_parent_class)->finalize (object);
 }
 
