@@ -48,6 +48,7 @@
 
 #include "gpm-common.h"
 #include "gpm-stock-icons.h"
+#include "gpm-power.h"
 #include "gpm-tray-icon.h"
 #include "gpm-debug.h"
 
@@ -57,29 +58,25 @@ static void     gpm_tray_icon_finalize   (GObject	   *object);
 
 #define GPM_TRAY_ICON_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), GPM_TYPE_TRAY_ICON, GpmTrayIconPrivate))
 
+#define MAX_BATTERIES_PER_TYPE 5 /* the cosmetic restriction on the dropdown */
+
 struct GpmTrayIconPrivate
 {
-	GtkUIManager	*ui_manager;
-	GtkActionGroup	*actiongroup;
-	GtkWidget	*popup_menu;
-	GtkStatusIcon	*status_icon;
-
-	gboolean	 show_notifications;
-	gboolean	 is_visible;
-
-	gboolean	 can_suspend;
-	gboolean	 can_hibernate;
-
-	gchar		*stock_id;
+	GtkStatusIcon		*status_icon;
+	GpmPower		*power;
+	gboolean		 show_notifications;
+	gboolean		 is_visible;
+	gboolean		 show_suspend;
+	gboolean		 show_hibernate;
+	gchar			*stock_id;
 #ifdef HAVE_LIBNOTIFY
-	NotifyNotification *notify;
+	NotifyNotification	*notify;
 #endif
 };
 
 enum {
 	SUSPEND,
 	HIBERNATE,
-	SHOW_INFO,
 	LAST_SIGNAL
 };
 
@@ -88,34 +85,14 @@ enum {
 	PROP_MODE
 };
 
-static void gpm_tray_icon_suspend_cb		(GtkAction *action, GpmTrayIcon *icon);
-static void gpm_tray_icon_hibernate_cb		(GtkAction *action, GpmTrayIcon *icon);
-static void gpm_tray_icon_show_info_cb	 	(GtkAction *action, GpmTrayIcon *icon);
-static void gpm_tray_icon_show_preferences_cb	(GtkAction *action, GpmTrayIcon *icon);
-static void gpm_tray_icon_show_statistics_cb		(GtkAction *action, GpmTrayIcon *icon);
-static void gpm_tray_icon_show_help_cb		(GtkAction *action, GpmTrayIcon *icon);
-static void gpm_tray_icon_show_about_cb		(GtkAction *action, GpmTrayIcon *icon);
-
-static GtkActionEntry gpm_tray_icon_action_entries [] =
-{
-	{ "TraySuspend", GPM_STOCK_SUSPEND, N_("_Suspend"),
-	  NULL, N_("Suspend the computer"), G_CALLBACK (gpm_tray_icon_suspend_cb) },
-	{ "TrayHibernate", GPM_STOCK_HIBERNATE, N_("Hi_bernate"),
-	  NULL, N_("Make the computer go to sleep"), G_CALLBACK (gpm_tray_icon_hibernate_cb) },
-	{ "TrayPreferences", GTK_STOCK_PREFERENCES, N_("_Preferences"),
-	  NULL, NULL, G_CALLBACK (gpm_tray_icon_show_preferences_cb) },
-	{ "TrayInfo", GTK_STOCK_DIALOG_INFO, N_("_Information"),
-	  NULL, NULL, G_CALLBACK (gpm_tray_icon_show_info_cb) },
-	{ "TrayStatistics", GTK_STOCK_CONVERT, N_("_Statistics"),
-	  NULL, NULL, G_CALLBACK (gpm_tray_icon_show_statistics_cb) },
-	{ "TrayHelp", GTK_STOCK_HELP, N_("_Help"), NULL,
-	  NULL, G_CALLBACK (gpm_tray_icon_show_help_cb) },
-	{ "TrayAbout", GTK_STOCK_ABOUT, N_("_About"), NULL,
-	  NULL, G_CALLBACK (gpm_tray_icon_show_about_cb) }
-};
-static guint gpm_tray_icon_n_action_entries = G_N_ELEMENTS (gpm_tray_icon_action_entries);
-
 static guint	 signals [LAST_SIGNAL] = { 0, };
+static gboolean
+libnotify_event (GpmTrayIcon    *icon,
+		 const gchar	*title,
+		 const gchar	*content,
+		 guint		 timeout,
+		 const gchar	*msgicon,
+		 GpmNotifyLevel	 urgency);
 
 G_DEFINE_TYPE (GpmTrayIcon, gpm_tray_icon, G_TYPE_OBJECT)
 
@@ -129,15 +106,7 @@ gpm_tray_icon_enable_suspend (GpmTrayIcon *icon,
 			      gboolean     enabled)
 {
 	g_return_if_fail (GPM_IS_TRAY_ICON (icon));
-
-	if (icon->priv->can_suspend != enabled) {
-		GtkAction *action;
-
-		icon->priv->can_suspend = enabled;
-		action = gtk_action_group_get_action (icon->priv->actiongroup,
-						      "TraySuspend");
-		gtk_action_set_visible (GTK_ACTION (action), enabled);
-	}
+	icon->priv->show_suspend = enabled;
 }
 
 /**
@@ -150,15 +119,7 @@ gpm_tray_icon_enable_hibernate (GpmTrayIcon *icon,
 				gboolean     enabled)
 {
 	g_return_if_fail (GPM_IS_TRAY_ICON (icon));
-
-	if (icon->priv->can_hibernate != enabled) {
-		GtkAction *action;
-
-		icon->priv->can_hibernate = enabled;
-		action = gtk_action_group_get_action (icon->priv->actiongroup,
-						      "TrayHibernate");
-		gtk_action_set_visible (GTK_ACTION (action), enabled);
-	}
+	icon->priv->show_hibernate = enabled;
 }
 
 /**
@@ -173,8 +134,7 @@ gpm_tray_icon_set_tooltip (GpmTrayIcon  *icon,
 	g_return_if_fail (GPM_IS_TRAY_ICON (icon));
 	g_return_if_fail (tooltip != NULL);
 
-	gtk_status_icon_set_tooltip (GTK_STATUS_ICON (icon->priv->status_icon),
-				     tooltip);
+	gtk_status_icon_set_tooltip (GTK_STATUS_ICON (icon->priv->status_icon), tooltip);
 }
 
 /**
@@ -212,11 +172,33 @@ gpm_tray_icon_set_image_from_stock (GpmTrayIcon *icon,
  * @icon: This TrayIcon class instance
  **/
 static void
-gpm_tray_icon_show_info_cb (GtkAction   *action,
-			    GpmTrayIcon *icon)
+gpm_tray_icon_show_info_cb (GtkMenuItem *item, gpointer data)
 {
-	gpm_debug ("emitting show_info");
-	g_signal_emit (icon, signals [SHOW_INFO], 0);
+	GpmTrayIcon *icon = GPM_TRAY_ICON (data);
+	const char *udi = g_object_get_data (G_OBJECT (item), "udi");
+	GpmPowerDevice *device;
+	char *msgicon;
+	char *desc;
+	char *longdesc;
+	GString *gdesc;
+
+	gpm_debug ("udi=%s", udi);
+	device = gpm_power_get_device_from_udi (icon->priv->power, udi);
+	if (device == NULL) {
+		return;
+	}
+
+	msgicon = gpm_power_get_icon_from_status (&device->battery_status, device->battery_kind);
+	gdesc = gpm_power_status_for_device (device);
+	desc = g_strdup (gdesc->str);
+	g_string_free (gdesc, TRUE);
+
+	/* get long description */
+	gdesc = gpm_power_status_for_device_more (device);
+	longdesc = g_strdup (gdesc->str);
+	g_string_free (gdesc, TRUE);
+
+	libnotify_event (icon, desc, longdesc, 0, msgicon, GPM_NOTIFY_URGENCY_LOW);
 }
 
 /**
@@ -225,9 +207,9 @@ gpm_tray_icon_show_info_cb (GtkAction   *action,
  * @icon: This TrayIcon class instance
  **/
 static void
-gpm_tray_icon_hibernate_cb (GtkAction   *action,
-			    GpmTrayIcon *icon)
+gpm_tray_icon_hibernate_cb (GtkMenuItem *item, gpointer data)
 {
+	GpmTrayIcon *icon = GPM_TRAY_ICON (data);
 	gpm_debug ("emitting hibernate");
 	g_signal_emit (icon, signals [HIBERNATE], 0);
 }
@@ -238,9 +220,9 @@ gpm_tray_icon_hibernate_cb (GtkAction   *action,
  * @icon: This TrayIcon class instance
  **/
 static void
-gpm_tray_icon_suspend_cb (GtkAction   *action,
-			  GpmTrayIcon *icon)
+gpm_tray_icon_suspend_cb (GtkMenuItem *item, gpointer data)
 {
+	GpmTrayIcon *icon = GPM_TRAY_ICON (data);
 	gpm_debug ("emitting suspend");
 	g_signal_emit (icon, signals [SUSPEND], 0);
 }
@@ -251,8 +233,7 @@ gpm_tray_icon_suspend_cb (GtkAction   *action,
  * @icon: This TrayIcon class instance
  **/
 static void
-gpm_tray_icon_show_statistics_cb (GtkAction   *action,
-				   GpmTrayIcon *icon)
+gpm_tray_icon_show_statistics_cb (GtkMenuItem *item, gpointer data)
 {
 	const gchar *command = "gnome-power-statistics";
 
@@ -267,8 +248,7 @@ gpm_tray_icon_show_statistics_cb (GtkAction   *action,
  * @icon: This TrayIcon class instance
  **/
 static void
-gpm_tray_icon_show_preferences_cb (GtkAction   *action,
-				   GpmTrayIcon *icon)
+gpm_tray_icon_show_preferences_cb (GtkMenuItem *item, gpointer data)
 {
 	const gchar *command = "gnome-power-preferences";
 
@@ -283,8 +263,7 @@ gpm_tray_icon_show_preferences_cb (GtkAction   *action,
  * @icon: This TrayIcon class instance
  **/
 static void
-gpm_tray_icon_show_help_cb (GtkAction   *action,
-			    GpmTrayIcon *icon)
+gpm_tray_icon_show_help_cb (GtkMenuItem *item, gpointer data)
 {
 	GError *error = NULL;
 
@@ -301,8 +280,7 @@ gpm_tray_icon_show_help_cb (GtkAction   *action,
  * @icon: This TrayIcon class instance
  **/
 static void
-gpm_tray_icon_show_about_cb (GtkAction   *action,
-			     GpmTrayIcon *icon)
+gpm_tray_icon_show_about_cb (GtkMenuItem *item, gpointer data)
 {
 	const char *authors[] = {
 		"Richard Hughes <richard@hughsie.com>",
@@ -370,30 +348,8 @@ static void
 gpm_tray_icon_popup_cleared_cd (GtkWidget   *widget,
 				GpmTrayIcon *icon)
 {
-	g_return_if_fail (GPM_IS_TRAY_ICON (icon));
-}
-
-/**
- * gpm_tray_icon_sync_actions:
- * @icon: This TrayIcon class instance
- *
- * Syncs the private icon->priv->can_* variables with the icon states
- **/
-static void
-gpm_tray_icon_sync_actions (GpmTrayIcon *icon)
-{
-	GtkAction *action;
-	g_return_if_fail (GPM_IS_TRAY_ICON (icon));
-
-	if (icon->priv->actiongroup != NULL) {
-		action = gtk_action_group_get_action (icon->priv->actiongroup,
-						      "TraySuspend");
-		gtk_action_set_visible (GTK_ACTION (action), icon->priv->can_suspend);
-
-		action = gtk_action_group_get_action (icon->priv->actiongroup,
-						      "TrayHibernate");
-		gtk_action_set_visible (GTK_ACTION (action), icon->priv->can_hibernate);
-	}
+//	g_return_if_fail (GPM_IS_TRAY_ICON (icon));
+	g_debug ("clear tray (icon = %p)", icon);
 }
 
 /**
@@ -408,8 +364,6 @@ gpm_tray_icon_constructor (GType		  type,
 {
 	GpmTrayIcon      *tray;
 	GpmTrayIconClass *klass;
-	GError		 *error = NULL;
-	GtkWidget	 *widget;
 
 	klass = GPM_TRAY_ICON_CLASS (g_type_class_peek (GPM_TYPE_TRAY_ICON));
 
@@ -417,46 +371,6 @@ gpm_tray_icon_constructor (GType		  type,
 			      (type, n_construct_properties,
 			       construct_properties));
 
-	tray->priv->actiongroup = gtk_action_group_new ("TrayActions");
-
-	gtk_action_group_set_translation_domain (tray->priv->actiongroup,
-						 GETTEXT_PACKAGE);
-
-	gtk_action_group_add_actions (tray->priv->actiongroup,
-				      gpm_tray_icon_action_entries,
-				      gpm_tray_icon_n_action_entries,
-				      tray);
-	gpm_tray_icon_sync_actions (tray);
-
-	gtk_ui_manager_insert_action_group (tray->priv->ui_manager, tray->priv->actiongroup, 0);
-	gtk_ui_manager_add_ui_from_string (tray->priv->ui_manager,
-					   "<ui>"
-					   "  <popup name=\"GpmTrayPopup\">"
-					   "    <menuitem action=\"TraySuspend\" />"
-					   "    <menuitem action=\"TrayHibernate\" />"
-					   "    <separator />"
-					   "    <menuitem action=\"TrayPreferences\" />"
-					   "    <menuitem action=\"TrayInfo\" />"
-/*					   "    <menuitem action=\"TrayStatistics\" />"*/
-					   "    <separator />"
-					   "    <menuitem action=\"TrayHelp\" />"
-					   "    <menuitem action=\"TrayAbout\" />"
-					   "  </popup>"
-					   "</ui>",
-					   -1, &error);
-	if (error != NULL) {
-		gpm_warning ("Couldn't merge user interface for popup: %s", error->message);
-		g_clear_error (&error);
-	}
-
-	gtk_ui_manager_ensure_update (tray->priv->ui_manager);
-
-	/* Get notified of when the menu goes, as we have to re-enable the tooltip */
-	widget = gtk_ui_manager_get_widget (tray->priv->ui_manager, "/GpmTrayPopup");
-	g_signal_connect (GTK_WIDGET (widget), "hide", G_CALLBACK (gpm_tray_icon_popup_cleared_cd), tray);
-
-	g_object_unref (tray->priv->actiongroup);
-
 	return G_OBJECT (tray);
 }
 
@@ -493,16 +407,6 @@ gpm_tray_icon_class_init (GpmTrayIconClass *klass)
 			      g_cclosure_marshal_VOID__VOID,
 			      G_TYPE_NONE,
 			      0);
-	signals [SHOW_INFO] =
-		g_signal_new ("show-info",
-			      G_TYPE_FROM_CLASS (object_class),
-			      G_SIGNAL_RUN_LAST,
-			      G_STRUCT_OFFSET (GpmTrayIconClass, hibernate),
-			      NULL,
-			      NULL,
-			      g_cclosure_marshal_VOID__VOID,
-			      G_TYPE_NONE,
-			      0);
 }
 
 /**
@@ -527,24 +431,118 @@ gpm_tray_icon_show (GpmTrayIcon *icon,
 
 /**
  * gpm_tray_icon_popup_menu_cb:
- * @button: Which buttons are pressed
- * @icon: This TrayIcon class instance
  *
  * Display the popup menu.
  **/
 static void
 gpm_tray_icon_popup_menu_cb (GtkStatusIcon *status_icon,
-			     guint          button,
-			     guint32        activate_time,
-			     GpmTrayIcon    *icon)
+			     GpmTrayIcon   *icon)
 {
-	GtkWidget *popup;
-	popup = gtk_ui_manager_get_widget (GTK_UI_MANAGER (icon->priv->ui_manager),
-					   "/GpmTrayPopup");
+	GtkMenu *menu = (GtkMenu*) gtk_menu_new ();
+	GtkWidget *item;
+	GtkWidget *image;
 
-	gtk_menu_popup (GTK_MENU (popup), NULL, NULL,
+	gpm_debug ("icon right clicked");
+
+	/* Preferences */
+	item = gtk_image_menu_item_new_with_mnemonic (_("_Preferences"));
+	image = gtk_image_new_from_icon_name (GTK_STOCK_PREFERENCES, GTK_ICON_SIZE_MENU);
+	gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (item), image);
+	g_signal_connect (G_OBJECT (item), "activate",
+			  G_CALLBACK (gpm_tray_icon_show_preferences_cb), icon);
+	gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
+
+	/* Statistics */
+	item = gtk_image_menu_item_new_with_mnemonic (_("Power _History"));
+	image = gtk_image_new_from_icon_name (GPM_STOCK_STATISTICS, GTK_ICON_SIZE_MENU);
+	gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (item), image);
+	g_signal_connect (G_OBJECT (item), "activate",
+			  G_CALLBACK (gpm_tray_icon_show_statistics_cb), icon);
+	gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
+
+	/* Separator for HIG? */
+	item = gtk_separator_menu_item_new ();
+	gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
+
+	/* Help */
+	item = gtk_image_menu_item_new_with_mnemonic (_("_Help"));
+	image = gtk_image_new_from_icon_name (GTK_STOCK_HELP, GTK_ICON_SIZE_MENU);
+	gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (item), image);
+	g_signal_connect (G_OBJECT (item), "activate",
+			  G_CALLBACK (gpm_tray_icon_show_help_cb), icon);
+	gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
+
+	/* About */
+	item = gtk_image_menu_item_new_with_mnemonic (_("_About"));
+	image = gtk_image_new_from_icon_name (GTK_STOCK_ABOUT, GTK_ICON_SIZE_MENU);
+	gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (item), image);
+	g_signal_connect (G_OBJECT (item), "activate",
+			  G_CALLBACK (gpm_tray_icon_show_about_cb), icon);
+	gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
+
+	/* show the menu */
+	gtk_widget_show_all (GTK_WIDGET (menu));
+	gtk_menu_popup (GTK_MENU (menu), NULL, NULL,
 			gtk_status_icon_position_menu, status_icon,
-			button, activate_time);
+			1, gtk_get_current_event_time());
+
+	g_signal_connect (GTK_WIDGET (menu), "hide",
+			  G_CALLBACK (gpm_tray_icon_popup_cleared_cd), icon);
+}
+
+/**
+ * gpm_tray_icon_add_device:
+ *
+ * Add all the selected type of devices to the menu to form "drop down" info.
+ **/
+static guint
+gpm_tray_icon_add_device (GpmTrayIcon *icon,
+			  GtkMenu     *menu,
+			  GpmPowerKind kind)
+{
+	GtkWidget *item;
+	GtkWidget *image;
+	GpmPowerDevice *device;
+	guint i;
+	gchar *icon_name;
+	gchar *label;
+	const gchar *desc;
+	gint percentage;
+
+	for (i=0; i<MAX_BATTERIES_PER_TYPE; i++) {
+		device = gpm_power_get_battery_device_entry (icon->priv->power, kind, i);
+		if (device == NULL) {
+			break;
+		}
+
+		/* only add battery to list if present */
+		if (device->battery_status.is_present == FALSE) {
+			gpm_debug ("not adding device '%s' as not present", device->udi);
+			break;
+		}
+		gpm_debug ("adding device '%s'", device->udi);
+
+		/* generate the label */
+		percentage = device->battery_status.percentage_charge;
+		desc = gpm_power_kind_to_localised_string (kind);
+		label = g_strdup_printf ("%s (%i%%)", desc, percentage);
+		item = gtk_image_menu_item_new_with_label (label);
+		g_free (label);
+
+		/* generate the image */
+		icon_name = gpm_power_get_icon_from_status (&device->battery_status, kind);
+		image = gtk_image_new_from_icon_name (icon_name, GTK_ICON_SIZE_MENU);
+		gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (item), image);
+		g_free (icon_name);
+
+		/* callback with the UDI and add the the menu */
+		g_signal_connect (G_OBJECT (item), "activate",
+				  G_CALLBACK (gpm_tray_icon_show_info_cb), icon);
+		g_object_set_data (G_OBJECT (item), "udi", (gpointer) device->udi);
+		gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
+
+	}
+	return i;
 }
 
 /**
@@ -558,7 +556,52 @@ static void
 gpm_tray_icon_activate_cb (GtkStatusIcon *status_icon,
 			   GpmTrayIcon   *icon)
 {
-	gpm_debug ("icon activated");
+	GtkMenu *menu = (GtkMenu*) gtk_menu_new ();
+	GtkWidget *item;
+	GtkWidget *image;
+	guint dev_cnt = 0;
+
+	gpm_debug ("icon left clicked");
+
+	/* add all device types to the drop down menu */
+	dev_cnt += gpm_tray_icon_add_device (icon, menu, GPM_POWER_KIND_PRIMARY);
+	dev_cnt += gpm_tray_icon_add_device (icon, menu, GPM_POWER_KIND_UPS);
+	dev_cnt += gpm_tray_icon_add_device (icon, menu, GPM_POWER_KIND_MOUSE);
+
+	/* only do the seporator if we have at least one device */
+	if (dev_cnt != 0) {
+		item = gtk_separator_menu_item_new ();
+		gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
+	}
+
+	/* Suspend if available */
+	if (icon->priv->show_suspend) {
+		item = gtk_image_menu_item_new_with_mnemonic (_("_Suspend"));
+		image = gtk_image_new_from_icon_name (GPM_STOCK_SUSPEND, GTK_ICON_SIZE_MENU);
+		gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (item), image);
+		g_signal_connect (G_OBJECT (item), "activate",
+				  G_CALLBACK (gpm_tray_icon_suspend_cb), icon);
+		gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
+	}
+
+	/* Hibernate if available */
+	if (icon->priv->show_hibernate) {
+		item = gtk_image_menu_item_new_with_mnemonic (_("Hi_bernate"));
+		image = gtk_image_new_from_icon_name (GPM_STOCK_HIBERNATE, GTK_ICON_SIZE_MENU);
+		gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (item), image);
+		g_signal_connect (G_OBJECT (item), "activate",
+				  G_CALLBACK (gpm_tray_icon_hibernate_cb), icon);
+		gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
+	}
+
+	/* show the menu */
+	gtk_widget_show_all (GTK_WIDGET (menu));
+	gtk_menu_popup (GTK_MENU (menu), NULL, NULL,
+			gtk_status_icon_position_menu, status_icon,
+			1, gtk_get_current_event_time());
+
+	g_signal_connect (GTK_WIDGET (menu), "hide",
+			  G_CALLBACK (gpm_tray_icon_popup_cleared_cd), icon);
 }
 
 /**
@@ -577,8 +620,7 @@ gpm_tray_icon_init (GpmTrayIcon *icon)
 	/* FIXME: make this a property */
 	icon->priv->show_notifications = TRUE;
 	icon->priv->stock_id = g_strdup ("about-blank");
-
-	icon->priv->ui_manager = gtk_ui_manager_new ();
+	icon->priv->power = gpm_power_new ();
 
 	icon->priv->status_icon = gtk_status_icon_new ();
 	g_signal_connect_object (G_OBJECT (icon->priv->status_icon),
@@ -614,6 +656,19 @@ gpm_tray_icon_finalize (GObject *object)
 
 	tray_icon = GPM_TRAY_ICON (object);
 
+	if (tray_icon->priv->stock_id != NULL) {
+		g_free (tray_icon->priv->stock_id);
+	}
+	if (tray_icon->priv->power != NULL) {
+		g_object_unref (tray_icon->priv->power);
+	}
+	if (tray_icon->priv->status_icon != NULL) {
+		g_object_unref (tray_icon->priv->status_icon);
+	}
+	if (tray_icon->priv->notify != NULL) {
+		notify_notification_close (tray_icon->priv->notify, NULL);
+	}
+
 	g_return_if_fail (tray_icon->priv != NULL);
 
 	G_OBJECT_CLASS (gpm_tray_icon_parent_class)->finalize (object);
@@ -671,12 +726,12 @@ libnotify_event (GpmTrayIcon    *icon,
 		notify_notification_close (icon->priv->notify, NULL);
 	}
 
-#if 0
-	/* we can't point because of a bug in libnotify. Need dependency on 0.4.3 */
+/* DISTROS: If you've patched your libnotify 0.4.2 package you can remove this check */
+#if HAVE_LIBNOTIFY_NEW
 	icon->priv->notify = notify_notification_new_with_status_icon (title, content,
 								       msgicon, icon->priv->status_icon);
 #else
-	/* Use the bottom of the screen */
+	/* we can't point because of a bug in libnotify. Need dependency on 0.4.3 */
 	icon->priv->notify = notify_notification_new (title, content,
 						      msgicon, NULL);
 #endif
@@ -795,7 +850,7 @@ gpm_tray_icon_cancel_notify (GpmTrayIcon *icon)
 
 #ifdef HAVE_LIBNOTIFY
 	if (icon->priv->notify != NULL) {
-		notify_notification_close (icon->priv->notify, &error);
+		g_object_unref (icon->priv->notify);
 	}
 	if (error != NULL) {
 		g_error_free (error);
